/**
 *******************************************************************************
 * @file    tmpm440_adc.c
 * @brief   This file provides API functions for ADC driver.
 * @version V2.0.2.3
 * @date    2014/04/29
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm440_adc.h"

#if defined(__TMPM440_ADC_H)

/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @defgroup ADC 
  * @brief ADC driver modules
  * @{
  */

/** @defgroup ADC_Private_Defines
  * @{
  */
#define MOD6_ADRST_10           ((uint8_t)0x02U)
#define MOD6_ADRST_01           ((uint8_t)0x01U)

#define MOD3_ITM_CLEAR          ((uint8_t)0x03U)

#define MOD2_ADCH_CLEAR         ((uint8_t)0xF0U)
#define MOD2_HPADCH_CLEAR       ((uint8_t)0x0FU)

#define MOD1_ADHWE_CLEAR        ((uint8_t)0xEEU)
#define MOD1_ADHWE_SET          ((uint8_t)0x01U)
#define MOD1_ADHWS_CLEAR        ((uint8_t)0xEDU)
#define MOD1_HPADHWE_CLEAR      ((uint8_t)0xEBU)
#define MOD1_HPADHWE_SET        ((uint8_t)0x04U)
#define MOD1_HPADHWS_CLEAR      ((uint8_t)0xE7U)
#define MOD7_MASK               ((uint32_t)0x00000003)

/** @} */
/* End of group ADC_Private_Defines */

/** @defgroup ADC_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group ADC_Private_FunctionPrototypes */

/** @defgroup ADC_Private_Functions
  * @{
  */

/** @} */
/* End of group ADC_Private_Functions */

/** @defgroup ADC_Exported_Functions
  * @{
  */

/**
  * @brief  Software reset ADC.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @retval None.
  */
void ADC_SWReset(TSB_AD_TypeDef * ADx)
{
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    ADC_SetVref(ADx, ENABLE);
    ADx->MOD6 = MOD6_ADRST_10;
    ADx->MOD6 = MOD6_ADRST_01;
}

/**
  * @brief  Set AD sample hold time and prescaler clock.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @param  Sample_HoldTime: Select the AD sample hold time.
  *   This parameter can be one of the following values:
  *   ADC_CONVERSION_CLK_10, ADC_CONVERSION_CLK_20,
  *   ADC_CONVERSION_CLK_30, ADC_CONVERSION_CLK_40,
  *   ADC_CONVERSION_CLK_80, ADC_CONVERSION_CLK_160,
  *   ADC_CONVERSION_CLK_320.
  * @param  Prescaler_Output: Select the AD prescaler clock.
  *   This parameter can be one of the following values:
  *   ADC_FC_DIVIDE_LEVEL_2, ADC_FC_DIVIDE_LEVEL_4
  *   ADC_FC_DIVIDE_LEVEL_8, ADC_FC_DIVIDE_LEVEL_16.
  * @retval None.
  */
void ADC_SetClk(TSB_AD_TypeDef * ADx, uint32_t Sample_HoldTime, uint32_t Prescaler_Output)
{
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_HOLD_TIME(Sample_HoldTime));
    assert_param(IS_ADC_PRESCALER(Prescaler_Output));
    /* Set ADxCLK */
    ADx->CLK = Sample_HoldTime + Prescaler_Output;
}

/**
  * @brief  Start AD conversion.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @retval None.
  */
void ADC_Start(TSB_AD_TypeDef * ADx)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));

    /* Set ADxMOD0<ADS> = 1 to start AD conversion */
    tmp = ADx->MOD0;
    tmp |= 0x01U;
    ADx->MOD0 = tmp;
}

/**
  * @brief  Enable or disable ADC scan mode.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @param  NewState: Specify ADC scan mode state.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.
  * @retval None.
  */
void ADC_SetScanMode(TSB_AD_TypeDef * ADx, FunctionalState NewState)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    /* Set ADxMOD3<SCAN> */
    tmp = ADx->MOD3;
    if (NewState == ENABLE) {
        tmp |= 0x01U;
    } else {
        tmp &= ~(0x01U);
    }
    ADx->MOD3 = tmp;
}

/**
  * @brief  Enable or disable ADC repeat mode.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @param  NewState: Specify ADC repeat mode state.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.
  * @retval None.
  */
void ADC_SetRepeatMode(TSB_AD_TypeDef * ADx, FunctionalState NewState)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    /* Set ADxMOD3<REPEAT> */
    tmp = ADx->MOD3;
    if (NewState == ENABLE) {
        tmp |= 0x01U << 1U;
    } else {
        tmp &= ~(0x01U << 1U);
    }
    ADx->MOD3 = tmp;
}

/**
  * @brief  Set ADC interrupt mode in fixed channel repeat conversion mode.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @param  INTMode: Specify AD conversion interrupt mode.
  *   This parameter can be one of the following values:
  *   ADC_INT_SINGLE,       ADC_INT_CONVERSION_2,
  *   ADC_INT_CONVERSION_3, ADC_INT_CONVERSION_4,
  *   ADC_INT_CONVERSION_5, ADC_INT_CONVERSION_6,
  *   ADC_INT_CONVERSION_7, ADC_INT_CONVERSION_8.
  * @retval None.
  */
void ADC_SetINTMode(TSB_AD_TypeDef * ADx, uint8_t INTMode)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_INT_MODE(INTMode));
    /* Set ADxMOD3<ITM[2:0]> */
    tmp = ADx->MOD3;
    tmp &= MOD3_ITM_CLEAR;
    tmp |= INTMode;
    ADx->MOD3 = tmp;
}

/**
  * @brief  Set ADC input channel.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @param  InputChannel: Analog input channel.
  *   This parameter can be one of the following values in unit A or unit B:
  *    ADC_AN_00, ADC_AN_01, ADC_AN_02, ADC_AN_03,
  *    ADC_AN_04, ADC_AN_05, ADC_AN_06, ADC_AN_07.
  *   This parameter can be one of the following values in unit C:
  *    ADC_AN_00, ADC_AN_01, ADC_AN_02, ADC_AN_03.
  * @retval None.
  */
void ADC_SetInputChannel(TSB_AD_TypeDef * ADx, uint8_t InputChannel)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));

    if ((ADx == TSB_ADA) || (ADx == TSB_ADB)) {
        assert_param(IS_UNIT_AB_INPUT_CHANNEL(InputChannel));
    } else {
        assert_param(IS_UNIT_C_INPUT_CHANNEL(InputChannel));
    }
    /* Set ADxMOD2<ADCH[3:0]> */
    tmp = ADx->MOD2;
    tmp &= MOD2_ADCH_CLEAR;
    tmp |= InputChannel;
    ADx->MOD2 = tmp;
}



/**
  * @brief  Set ADC scan channel.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @param  StartChannel: Specify the start channel to be scanned.
  *   This parameter can be one of the following values in unit A or unit B:
  *    ADC_AN_00, ADC_AN_01, ADC_AN_02, ADC_AN_03,
  *    ADC_AN_04, ADC_AN_05, ADC_AN_06, ADC_AN_07.
  *   This parameter can be one of the following values in unit C:
  *    ADC_AN_00, ADC_AN_01, ADC_AN_02, ADC_AN_03.
  * @param  Range: Specify the range of assignable channel scan value.
  *   This parameter can be 1 to 8 in unit A or unit B and be 1 to 4 in unit C.
  * @retval None.
  */
void ADC_SetScanChannel(TSB_AD_TypeDef * ADx, uint8_t StartChannel, uint8_t Range)
{
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));

    if ((ADx == TSB_ADA) || (ADx == TSB_ADB)) {
        assert_param(IS_UNIT_AB_SCAN_CHANNEL(StartChannel, Range));
    } else {
        assert_param(IS_UNIT_C_SCAN_CHANNEL(StartChannel, Range));
    }
    /* Set ADxMOD4 */
    ADx->MOD4 = ((uint32_t) StartChannel | (((uint32_t) Range - 1U) << 4U));
}

/**
  * @brief  Control AVREFH-AVREFL current.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @param  VrefCtrl: Specify how to apply AVREFH-AVREFL current.
  *   This parameter can be one of the following values:
  *   ADC_APPLY_VREF_IN_CONVERSION or ADC_APPLY_VREF_AT_ANY_TIME.
  * @retval None.
  */
void ADC_SetVrefCut(TSB_AD_TypeDef * ADx, uint8_t VrefCtrl)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_VREF_CTRL(VrefCtrl));
    /* Set ADxMOD1<RCUT> */
    tmp = ADx->MOD1;
    if (VrefCtrl == ADC_APPLY_VREF_AT_ANY_TIME) {
        tmp |= 0x01U << 5U;
    } else {
        tmp &= ~(0x01U << 5U);
    }
    ADx->MOD1 = tmp;
}

/**
  * @brief  Set ADC operation in IDLE mode.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @param  NewState: Specify ADC operation state in IDLE mode.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.
  * @retval None.
  */
void ADC_SetIdleMode(TSB_AD_TypeDef * ADx, FunctionalState NewState)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    /* Set ADxMOD1<I2AD> */
    tmp = ADx->MOD1;
    if (NewState == ENABLE) {
        tmp |= 0x01U << 6U;
    } else {
        tmp &= ~(0x01U << 6U);
    }
    ADx->MOD1 = tmp;
}

/**
  * @brief  Set ADC VREF application.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @param  NewState: Specify ADC Vref application state.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.
  * @retval None.
  */
void ADC_SetVref(TSB_AD_TypeDef * ADx, FunctionalState NewState)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    /* Set ADxMOD1<DACON> */
    tmp = ADx->MOD1;
    if (NewState == ENABLE) {
        tmp |= 0x01U << 7U;
    } else {
        tmp &= ~(0x01U << 7U);
    }
    ADx->MOD1 = tmp;
}

/**
  * @brief  Select ADC top-priority conversion analog input channel.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @param  TopInputChannel: Analog input channel for top-priority conversion.
  *   This parameter can be one of the following values in unit A or unit B:
  *    ADC_AN_00, ADC_AN_01, ADC_AN_02, ADC_AN_03,
  *    ADC_AN_04, ADC_AN_05, ADC_AN_06, ADC_AN_07.
  *   This parameter can be one of the following values in unit C:
  *    ADC_AN_00, ADC_AN_01, ADC_AN_02, ADC_AN_03.
  * @retval None.
  */
void ADC_SetInputChannelTop(TSB_AD_TypeDef * ADx, uint8_t TopInputChannel)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    if ((ADx == TSB_ADA) || (ADx == TSB_ADB)) {
        assert_param(IS_UNIT_AB_INPUT_CHANNEL(TopInputChannel));
    } else {
        assert_param(IS_UNIT_C_INPUT_CHANNEL(TopInputChannel));
    }
    /* Set ADxMOD2<HPADCH[3:0]> */
    tmp = ADx->MOD2;
    tmp &= MOD2_HPADCH_CLEAR;
    tmp |= ((uint32_t) TopInputChannel << 4U);
    ADx->MOD2 = tmp;
}

/**
  * @brief  Start top-priority AD conversion.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @retval None.
  */
void ADC_StartTopConvert(TSB_AD_TypeDef * ADx)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    /* Set ADxMOD0<HPADS> = 1 to start top-priority AD conversion */
    tmp = ADx->MOD0;
    tmp |= 0x01U << 1U;
    ADx->MOD0 = tmp;
}

/**
  * @brief  Enable or disable the specified ADC monitor module.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @param  ADCMPx: Select which compare control register will be used.
  *   This parameter can be one of the following values:
  *   ADC_CMPCR_0 or ADC_CMPCR_1.
  * @param  NewState: Specify ADC monitor function state.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.
  * @retval None.
  */
void ADC_SetMonitor(TSB_AD_TypeDef * ADx, ADC_CMPCRx ADCMPx, FunctionalState NewState)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_CMPCRx(ADCMPx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    if (ADCMPx == ADC_CMPCR_0) {
        /* Set ADxCMPCR0<CMP0EN> */
        tmp = ADx->CMPCR0;
        if (NewState == ENABLE) {
            tmp |= 0x01U << 7U;
        } else {
            tmp &= ~(0x01U << 7U);
        }
        ADx->CMPCR0 = tmp;
    } else {
        /* Set ADxCMPCR1<CMP1EN> */
        tmp = ADx->CMPCR1;
        if (NewState == ENABLE) {
            tmp |= 0x01U << 7U;
        } else {
            tmp &= ~(0x01U << 7U);
        }
        ADx->CMPCR1 = tmp;
    }
}

/**
  * @brief  Configure the specified ADC monitor module.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @param  ADCMPx: Select which compare control register will be used.
  *   This parameter can be one of the following values:
  *   ADC_CMPCR_0 or ADC_CMPCR_1.
  * @param  Monitor: The structure containing ADC monitor configuration.
  * @retval None.
  */
void ADC_ConfigMonitor(TSB_AD_TypeDef * ADx, ADC_CMPCRx ADCMPx, ADC_MonitorTypeDef * Monitor)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_CMPCRx(ADCMPx));
    assert_param(IS_POINTER_NOT_NULL(Monitor));
    if ((ADx == TSB_ADA) || (ADx == TSB_ADB)) {
        assert_param(IS_UNIT_AB_INPUT_CHANNEL(Monitor->CmpChannel));
    } else {
        assert_param(IS_UNIT_C_INPUT_CHANNEL(Monitor->CmpChannel));
    }
    assert_param(IS_ADC_CMPCNT(Monitor->CmpCnt));
    assert_param(IS_ADC_CMPCONDITION(Monitor->Condition));
    assert_param(IS_ADC_CMPMODE(Monitor->CntMode));
    assert_param(IS_ADC_CMPVALUE_12BIT(Monitor->CmpValue));

    tmp |= (uint32_t) (Monitor->CmpChannel);
    tmp |= (uint32_t) (Monitor->Condition) << 4U;
    tmp |= (uint32_t) (Monitor->CntMode) << 5U;
    tmp |= (uint32_t) (Monitor->CmpCnt - 1U) << 8U;

    if (ADCMPx == ADC_CMPCR_0) {
        ADx->CMPCR0 = tmp;
        ADx->CMP0 = Monitor->CmpValue;
    } else {
        ADx->CMPCR1 = tmp;
        ADx->CMP1 = Monitor->CmpValue;
    }
}

/**
  * @brief  Set hardware trigger for normal AD conversion.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @param  HwSource: Hardware source for activating normal AD conversion.
  *   This parameter can be one of the following values:
  *   ADC_EXT_TRG or ADC_MATCH_TMRB_NORMAL.
  *   ADC_EXT_TRG include: ADTRGx and ADTRGSNC(x can be A, B, C).
  *   It can be started AD conversion of Unit A and UnitB at same time by ADGGSNC (PH3) pin.
  *   ADGGSNC (PH3) pin is only used in Unit A and Unit B.
  *   The Timer is TB17RG0 for normal AD conversion while in Unit A.
  *   The Timer is TB18RG0 for normal AD conversion while in Unit B.
  *   The Timer is TB19RG0 for normal AD conversion while in Unit C.
  * @param   NewState: Specify state of hardware source for activating normal AD conversion.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.
  * @retval None.
  */
void ADC_SetHWTrg(TSB_AD_TypeDef * ADx, uint8_t HwSource, FunctionalState NewState)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_HW_TRG_NORMAL(HwSource));
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    /* Set ADxMOD1<ADHWS> */
    tmp = ADx->MOD1;
    tmp &= MOD1_ADHWS_CLEAR;
    tmp |= HwSource;
    /* Set ADxMOD1<ADHWE> */
    if (NewState == ENABLE) {
        tmp |= MOD1_ADHWE_SET;
    } else {
        tmp &= MOD1_ADHWE_CLEAR;
    }
    ADx->MOD1 = tmp;
}

/**
  * @brief  Set hardware trigger for top-priority AD conversion.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @param  HwSource: Hardware source for activating top-priority AD conversion.
  *   This parameter can be one of the following values:
  *   ADC_EXT_TRG or ADC_MATCH_TMRB_TOP.
  *   ADC_EXT_TRG include: ADTRGx and ADTRGSNC(x can be A, B, C).
  *   It can be started AD conversion of Unit A and UnitB at same time by ADGGSNC (PH3) pin.
  *   ADGGSNC (PH3) pin is only used in Unit A and Unit B.
  *   The Timer is TB14RG0 for top-priority AD conversion while in Unit A.
  *   The Timer is TB15RG0 for top-priority AD conversion while in Unit B.
  *   The Timer is TB16RG0 for top-priority AD conversion while in Unit C.
  * @param   NewState: Specify state of hardware source for activating top-priority AD conversion.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.
  * @retval None.
  */
void ADC_SetHWTrgTop(TSB_AD_TypeDef * ADx, uint8_t HwSource, FunctionalState NewState)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_HW_TRG_TOP(HwSource));
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    /* Set ADxMOD1<HPADHWS> */
    tmp = ADx->MOD1;
    tmp &= MOD1_HPADHWS_CLEAR;
    tmp |= HwSource;
    /* Set ADxMOD1<HPADHWE> */
    if (NewState == ENABLE) {
        tmp |= MOD1_HPADHWE_SET;
    } else {
        tmp &= MOD1_HPADHWE_CLEAR;
    }
    ADx->MOD1 = tmp;
}

/**
  * @brief  Read AD conversion completion flag (normal and top-priority).
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @retval A union with the state of AD conversion.
  */
ADC_State ADC_GetConvertState(TSB_AD_TypeDef * ADx)
{
    ADC_State retval = { 0U };
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    /* Read AD conversion completion flag */
    retval.All = ADx->MOD5;
    return retval;
}

/**
  * @brief  Read AD conversion result.
  * @param ADx:  Select ADC unit.
  *   This parameter can be one of the following values:
  *    TSB_ADA, TSB_ADB, TSB_ADC
  * @param  ADREGx: ADC result register.
  *   This parameter can be one of the following values:
  *   ADC_REG_00, ADC_REG_01, ADC_REG_02, ADC_REG_03,
  *   ADC_REG_04, ADC_REG_05, ADC_REG_06, ADC_REG_07, 
  *   ADC_REG_SP.
  * @retval A union with AD result and 3 bits of extra information.
  */
ADC_Result ADC_GetConvertResult(TSB_AD_TypeDef * ADx, uint8_t ADREGx)
{
    ADC_Result retval = { 0U };

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_REG(ADREGx));
    /* Read ADxREGx to get ADC result */
    switch (ADREGx) {
    case ADC_REG_00:
        retval.All = ADx->REG00;
        break;
    case ADC_REG_01:
        retval.All = ADx->REG01;
        break;
    case ADC_REG_02:
        retval.All = ADx->REG02;
        break;
    case ADC_REG_03:
        retval.All = ADx->REG03;
        break;
    case ADC_REG_04:
        retval.All = ADx->REG04;
        break;
    case ADC_REG_05:
        retval.All = ADx->REG05;
        break;
    case ADC_REG_06:
        retval.All = ADx->REG06;
        break;
    case ADC_REG_07:
        retval.All = ADx->REG07;
        break;
    case ADC_REG_SP:
        retval.All = ADx->REGSP;
        break;
    default:                   /* Do nothing */
        break;
    }

    return retval;
}

/**
  * @brief  Enable or disable DMA activation factor for normal or top-priority AD conversion.
  * @param  ADx: Select ADC unit.
  *   This parameter can be one of the following values:
  *   TSB_ADA, TSB_ADB, TSB_ADC  
  * @param  DMAReq: Specify AD conversion DMA request type.
  *   This parameter can be one of the following values:
  *   ADC_DMA_REQ_NORMAL:0, normal AD conversion,
  *   ADC_DMA_REQ_TOP:   1, top-priority AD conversion.
  * @param  NewState: Specify AD conversion DMA activation factor.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.
  * @retval None.
  */
void ADC_SetDMAReq(TSB_AD_TypeDef * ADx, uint8_t DMAReq, FunctionalState NewState)
{
    uint32_t reg = ADx->MOD7;
    /* Check the parameters */
    assert_param(IS_ADC_DMA_REQ(DMAReq));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    reg &= ~(1UL << DMAReq);
    reg &= MOD7_MASK;
    reg |= (uint32_t) NewState << DMAReq;
    ADx->MOD7 = reg;
}

/** @} */
/* End of group ADC_Exported_Functions */

/** @} */
/* End of group ADC */

/** @} */
/* End of group TX04_Periph_Driver */

#endif                          /* defined(__TMPM440_ADC_H) */
