/**
 *******************************************************************************
 * @file    tmpm440_ephc.c
 * @brief   This file provides API functions for EPHC driver.
 * @version V2.0.2.2
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm440_ephc.h"

#if defined (__TMPM440_EPHC_H)
/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @defgroup EPHC 
  * @brief EPHC driver modules
  * @{
  */
/** @defgroup EPHC_Private_Defines
  * @{
  */

#define EN_EPHCEN_SET        ((uint32_t)0x00000001)
#define EN_EPHCEN_CLEAR      ((uint32_t)0xFFFFFFFE)

#define CNT_ALLBITS_CLEAR    ((uint32_t)0xFFFF0488)
#define ARUN_CNT_CLEAR       ((uint32_t)0xFFFFFFFD)
#define EPHC_FLG_MASK        ((uint32_t)0x000001FF)

#define ARUN_RUN_SET         ((uint32_t)0x00000001)
#define ARUN_RUN_CLEAR       ((uint32_t)0xFFFFFFFE)

#define BRUN_RUN_SET         ((uint32_t)0x00000001)
#define BRUN_RUN_CLEAR       ((uint32_t)0xFFFFFFFE)

#define BDMA_DMADT0_SET      ((uint32_t)0x00000001)
#define BDMA_DMADT0_CLEAR    ((uint32_t)0xFFFFFFFE)

#define BCAP_VALUE_MASK      ((uint32_t)0x00FFFFFF)
#define BCAP_OVERFLOW_MASK   ((uint32_t)0x01000000)


/** @} */
/* End of group EPHC_Private_Defines */

/** @defgroup EPHC_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group EPHC_Private_FunctionPrototypes */

/** @defgroup EPHC_Private_Functions
  * @{
  */

/** @} */
/* End of group group EPHC_Private_Functions */

/** @defgroup EPHC_Exported_Functions
  * @{
  */

/**
  * @brief  Enable the specified EPHC channel.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @retval None
  */
void EPHC_Enable(TSB_EPHC_TypeDef * EPHCx)
{
    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));

    /* Set EPHCxEN<EN> to enable EPHCx */
    EPHCx->EN |= EN_EPHCEN_SET;
}

/**
  * @brief  Disable the specified EPHC channel.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC .
  * @retval None
  */
void EPHC_Disable(TSB_EPHC_TypeDef * EPHCx)
{
    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));

    /* Clear EPHCxEN<EN> to disable EPHCx */
    EPHCx->EN &= EN_EPHCEN_CLEAR;
}

/**
  * @brief  Initialize the specified EPHC channel.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @param  InitStruct: The structure containing basic EPHC configuration.
  * @retval None
  */
void EPHC_Init(TSB_EPHC_TypeDef * EPHCx, EPHC_InitTypeDef * InitStruct)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_POINTER_NOT_NULL(InitStruct));
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));
    assert_param(IS_EPHC_CNT_MA1DN(InitStruct->CountDownEdgeSelForP1));
    assert_param(IS_EPHC_CNT_MA1UP(InitStruct->CountUpEdgeSelForP1));
    assert_param(IS_EPHC_CNT_BRCK(InitStruct->InputClkSelection));
    assert_param(IS_EPHC_CNT_PBDIR_PHASESEL(InitStruct->PhaseSelection));
    assert_param(IS_EPHC_CNT_MA12_PULSEMODE(InitStruct->ModeSetting));
    assert_param(IS_EPHC_CNT_MA2DIR_DIRSETTING(InitStruct->DirectionSetting));
    assert_param(IS_EPHC_CNT_NOISEFILTER(InitStruct->NoiseFilterCtrl));
    assert_param(IS_EPHC_COUNT_CTRL(InitStruct->CountClearCtrl));

    if ((EPHCx->ARUN & EPHC_RUN) == 0U) {       /* stopping? */
        /* Configure mode for EPHCx */
        tmp = EPHCx->CNT;
        tmp &= CNT_ALLBITS_CLEAR;
        tmp |= InitStruct->CountDownEdgeSelForP1;
        tmp |= InitStruct->CountUpEdgeSelForP1;
        tmp |= InitStruct->InputClkSelection;
        tmp |= InitStruct->PhaseSelection;
        tmp |= InitStruct->ModeSetting;
        tmp |= InitStruct->DirectionSetting;
        tmp |= InitStruct->NoiseFilterCtrl;

        EPHCx->CNT = tmp;

        /* Control 16-bit counter clear for EPHCx */
        tmp = EPHCx->ARUN;
        tmp &= ARUN_CNT_CLEAR;
        tmp |= InitStruct->CountClearCtrl;
        EPHCx->ARUN = tmp;
    } else {
        /* Do nothing */
    }
}

/**
  * @brief  Enable some EPHC interrupt.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @param  EnableINT: Select the enable of EPHC interrupt.
  *   This parameter can be one of the following values:
  *   EPHC_IE_INT_PCCP0, EPHC_IE_INT_PCCP1,
  *   EPHC_IE_INT_PCOVF,EPHC_IE_INT_PCUDF,
  *   EPHC_IE_INT_PCDT0,EPHC_IE_INT_PCDT1,
  *   EPHC_IE_INT_PCDT2,EPHC_IE_INT_PCDT3,
  *   EPHC_IE_INT_PCDIR, EPHC_IE_INT_PCUOVF,
  *   EPHC_FLG_ALL.
  * @retval None
  */
void EPHC_EnableInterrupt(TSB_EPHC_TypeDef * EPHCx, uint32_t EnableINT)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));
    assert_param(IS_EPHC_INT_ENABLE(EnableINT));

    /* Enable the specified interrupt */
    tmp = EPHCx->IE;
    tmp |= EnableINT;
    EPHCx->IE = tmp;
}

/**
  * @brief  Disable some EPHC interrupt.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @param  DisableINT: Select the disable of EPHC interrupt.
  *   This parameter can be one of the following values:
  *   EPHC_IE_INT_PCCP0, EPHC_IE_INT_PCCP1,
  *   EPHC_IE_INT_PCOVF,EPHC_IE_INT_PCUDF,
  *   EPHC_IE_INT_PCDT0,EPHC_IE_INT_PCDT1,
  *   EPHC_IE_INT_PCDT2,EPHC_IE_INT_PCDT3,
  *   EPHC_IE_INT_PCDIR, EPHC_IE_INT_PCUOVF,
  *   EPHC_FLG_ALL.
  * @retval None
  */
void EPHC_DisableInterrupt(TSB_EPHC_TypeDef * EPHCx, uint32_t DisableINT)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));
    assert_param(IS_EPHC_INT_DISABLE(DisableINT));

    /* Mask the specified interrupt */
    tmp = EPHCx->IE;
    tmp &= (uint32_t) (~DisableINT);
    EPHCx->IE = tmp;
}

/**
  * @brief  Indicate what causes the interrupt.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @retval The interrupt factor of EPHCx.
  */
EPHC_INTFactor EPHC_GetINTFactor(TSB_EPHC_TypeDef * EPHCx)
{
    EPHC_INTFactor retval = { 0U };

    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));

    /* Get interrupt factor */
    retval.All = EPHCx->FLG & EPHC_FLG_MASK;
    return retval;
}

/**
  * @brief  Clear specified interrupt factor flag
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @param  ClearINT: Select the clear of EPHC interrupt.
  *   This parameter can be 1U to 1FFU.
  * @retval None
  */
void EPHC_ClearINTFactor(TSB_EPHC_TypeDef * EPHCx, uint32_t ClearINT)
{
    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));
    assert_param(IS_EPHC_FLG(ClearINT));

    /* Clear the specified interrupt */
    EPHCx->FLG |= ClearINT;
}

/**
  * @brief  Start or stop 16-bit counter of the specified EPHC channel.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @param  Cmd: The command for the counter.
  *   This parameter can be EPHC_RUN or EPHC_STOP.
  * @retval None
  */
void EPHC_ASetRunState(TSB_EPHC_TypeDef * EPHCx, uint8_t Cmd)
{
    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));
    assert_param(IS_EPHC_CMD(Cmd));

    /* Write command into ARUN register */
    EPHCx->ARUN = Cmd;
}

/**
  * @brief  Clear Pulse-counter value of EPHCx.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @retval None.
  */
void EPHC_AClearPulseCntValue(TSB_EPHC_TypeDef * EPHCx)
{
    uint32_t tmp = 0U;
    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));

    tmp = EPHCx->ARUN;
    if ((tmp & EPHC_RUN) == 0U) {       /* stopping? */
        /* Clear pulse-counter for EPHCx */
        tmp |= EPHC_COUNT_CLR;
        EPHCx->ARUN = tmp;
    } else {
        /* Do nothing */
    }
}

/**
  * @brief  Get EPHC compare value of EPHCx.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @param  CmpReg: Select the compare register to read.
  *   This parameter can be: EPHC_COMP_0 or EPHC_COMP_1.
  * @retval Capture value of EPHCx
  */
uint16_t EPHC_AGetCompareValue(TSB_EPHC_TypeDef * EPHCx, uint8_t CmpReg)
{
    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));
    assert_param(IS_EPHC_COMP_REG(CmpReg));

    /* Return compare value */
    return CmpReg ? (uint16_t) EPHCx->ACP1 : (uint16_t) EPHCx->ACP0;
}

/**
  * @brief  Set EPHC compare value of EPHCx.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @param  CmpReg: Select the compare register to read.
  *   This parameter can be: EPHC_COMP_0 or EPHC_COMP_1.
  * @param  CmpValue: the compare register value.
  *   This parameter can be: 0x0000 to 0xFFFF.
  * @retval None.
  */
void EPHC_ASetCompareValue(TSB_EPHC_TypeDef * EPHCx, uint8_t CmpReg, uint16_t CmpValue)
{
    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));
    assert_param(IS_EPHC_COMP_REG(CmpReg));
    /* assert_param(IS_EPHC_CMP_VALUE(CmpValue)); *//* uint16_t */

    /* Set compare register */
    if (CmpReg == EPHC_COMP_0) {
        EPHCx->ACP0 = (uint32_t) CmpValue;
    } else {
        EPHCx->ACP1 = (uint32_t) CmpValue;
    }
}

/**
  * @brief  Get Pulse-counter value of EPHCx.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @retval Pulse-counter value of EPHCx
  */
uint16_t EPHC_AGetPulseCntValue(TSB_EPHC_TypeDef * EPHCx)
{
    uint16_t regval = 0U;
    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));

    /* Return pulse-counter value */
    regval = (uint16_t) EPHCx->ADAT;

    return regval;
}

/**
  * @brief  Start or stop 24-bit counter of the specified EPHC channel.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @param  Cmd: The command for the counter.
  *   This parameter can be EPHC_BRUN or EPHC_BSTOP.
  * @retval None
  */
void EPHC_BSetRunState(TSB_EPHC_TypeDef * EPHCx, uint8_t Cmd)
{
    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));
    assert_param(IS_EPHC_BCMD(Cmd));

    /* Write command into BRUN register */
    EPHCx->BRUN = Cmd;
}

/**
  * @brief  Enable or disable DMA request.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @param  NewState: New state of DMA request.
  *   This parameter can be ENABLE or DISABLE.
  * @param  DMAReq: DMA request.
  *   This parameter can be EPHC_DMA_REQ_CAPTURE_0.
  * @retval None
  */
void EPHC_BSetDMAReq(TSB_EPHC_TypeDef * EPHCx, FunctionalState NewState, uint8_t DMAReq)
{
    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    assert_param(IS_EPHC_DMA_REQ(DMAReq));

    if (NewState == ENABLE) {
        EPHCx->BDMA = (uint32_t) DMAReq;
    } else {
        EPHCx->BDMA = (uint32_t) 0U;    /* clear directly */
    }
}

/**
  * @brief  24-bit counter read data for cycle phase measurement.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @retval 24-bit counter read data.
  */
uint32_t EPHC_BGetReadCntValue(TSB_EPHC_TypeDef * EPHCx)
{
    uint32_t regval = 0U;

    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));

    /* Read the counter */
    regval = EPHCx->BUC;

    /* Return the value */
    return regval;
}

/**
  * @brief  Capture register value.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @param  CapReg: Capture register.
  *   This parameter can be one of the following values:
  *   EPHC_BCAP00_IN0RISING,EPHC_BCAP10_IN0RISING,
  *   EPHC_BCAP30_IN0RISING,EPHC_BCAP30_IN0RISING.
  * @retval Capture register value.
  */
uint32_t EPHC_BGetCapRegValue(TSB_EPHC_TypeDef * EPHCx, uint8_t CapReg)
{
    uint32_t regval = 0U;

    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));
    assert_param(IS_EPHC_BCAP_REG(CapReg));

    /* Capture register value */
    switch (CapReg) {
    case EPHC_BCAP00_IN0RISING:
        regval = EPHCx->BCAP00;
        break;
    case EPHC_BCAP10_IN1RISING:
        regval = EPHCx->BCAP10;
        break;
    case EPHC_BCAP20_IN0FALLING:
        regval = EPHCx->BCAP20;
        break;
    case EPHC_BCAP30_IN1FALLING:
        regval = EPHCx->BCAP30;
        break;
    default:
        /* Do nothing */
        break;
    }
    regval &= BCAP_VALUE_MASK;
    /* Return the value */
    return regval;
}

/**
  * @brief  Overflow occurs or not.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @param  CapReg: Capture register.
  *   This parameter can be one of the following values:
  *   EPHC_BCAP00_IN0RISING,EPHC_BCAP10_IN0RISING,
  *   EPHC_BCAP30_IN0RISING,EPHC_BCAP30_IN0RISING.
  * @retval Overflow occurs or not.
  */
uint32_t EPHC_BGetCapRegOverflow(TSB_EPHC_TypeDef * EPHCx, uint8_t CapReg)
{
    uint32_t regval = 0U;

    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));
    assert_param(IS_EPHC_BCAP_REG(CapReg));

    /* Capture register value */
    switch (CapReg) {
    case EPHC_BCAP00_IN0RISING:
        regval = EPHCx->BCAP00;
        break;
    case EPHC_BCAP10_IN1RISING:
        regval = EPHCx->BCAP10;
        break;
    case EPHC_BCAP20_IN0FALLING:
        regval = EPHCx->BCAP20;
        break;
    case EPHC_BCAP30_IN1FALLING:
        regval = EPHCx->BCAP30;
        break;
    default:
        /* Do nothing */
        break;
    }
    regval &= BCAP_OVERFLOW_MASK;
    /* Return the value */
    return regval;
}

/**
  * @brief  Captures a value of cycle counter.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @param  CntReg: Cycle counter register.
  *   This parameter can be one of the following values:
  *   EPHC_B0DAT_IN0RISING,EPHC_B1DAT_IN1RISING,
  *   EPHC_B2DAT_IN0FALLING,EPHC_B3DAT_IN1FALLING,EPHC_BCDAT_COMMON.
  * @retval a value of cycle counter.
  */
uint32_t EPHC_BGetCycleCntRegValue(TSB_EPHC_TypeDef * EPHCx, uint8_t CntReg)
{
    uint32_t regval = 0U;

    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));
    assert_param(IS_EPHC_BDAT_REG(CntReg));

    /* Capture register value */
    switch (CntReg) {
    case EPHC_B0DAT_IN0RISING:
        regval = EPHCx->B0DAT;
        break;
    case EPHC_B1DAT_IN1RISING:
        regval = EPHCx->B1DAT;
        break;
    case EPHC_B2DAT_IN0FALLING:
        regval = EPHCx->B2DAT;
        break;
    case EPHC_B3DAT_IN1FALLING:
        regval = EPHCx->B3DAT;
        break;
    case EPHC_BCDAT_COMMON:
        regval = EPHCx->BCDAT;
        break;
    default:
        /* Do nothing */
        break;
    }
    /* Return the value */
    return regval;
}

/**
  * @brief  Phase Difference Register value.
  * @param  EPHCx: Select the EPHC channel.
  *   This parameter can be one of the following values:
  *   TSB_EPHC.
  * @param  CntReg: Cycle counter register.
  *   This parameter can be one of the following values:
  *   EPHC_B0PDT,EPHC_B1PDT,EPHC_B2PDT,EPHC_B3PDT.
  * @retval a value of cycle counter.
  */
uint32_t EPHC_BGetPhaseDiffRegValue(TSB_EPHC_TypeDef * EPHCx, uint8_t PhaseReg)
{
    uint32_t regval = 0U;

    /* Check the parameters */
    assert_param(IS_EPHC_ALL_PERIPH(EPHCx));
    assert_param(IS_EPHC_BPDT_REG(PhaseReg));

    /* Capture register value */
    switch (PhaseReg) {
    case EPHC_B0PDT:
        regval = EPHCx->B0PDT;
        break;
    case EPHC_B1PDT:
        regval = EPHCx->B1PDT;
        break;
    case EPHC_B2PDT:
        regval = EPHCx->B2PDT;
        break;
    case EPHC_B3PDT:
        regval = EPHCx->B3PDT;
        break;
    default:
        /* Do nothing */
        break;
    }
    /* Return the value */
    return regval;
}

/** @} */
/* End of group EPHC_Exported_Functions */

/** @} */
/* End of group EPHC */

/** @} */
/* End of group TX04_Periph_Driver */

#endif                          /* defined(__TMPM440_EPHC_H) */
