/**
 *******************************************************************************
 * @file    tmpm440_kwup.c
 * @brief   This file provides API functions for KWUP driver.
 * @version V2.0.2.2
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm440_kwup.h"

#if defined(__TMPM440_KWUP_H)

/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @defgroup KWUP 
  * @brief KWUP driver modules
  * @{
  */

/** @defgroup KWUP_Private_Defines
  * @{
  */
#define KWUP_CRn_MASK            ((uint32_t)0xFFFFFF0E)
#define KWUPB_PKEY_MASK          ((uint32_t)0x000000FF)
#define KWUPB_INT_MASK           ((uint32_t)0x000000FF)
#define KWUP_PULL_UP_CYCLE_MASK  ((uint32_t)0xFFFFFFC3)


/** @} */
/* End of group KWUP_Private_Defines */

/** @defgroup KWUP_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group KWUP_Private_FunctionPrototypes */

/** @defgroup KWUP_Private_Functions
  * @{
  */

/** @} */
/* End of group KWUP_Private_Functions */

/** @defgroup KWUP_Exported_Functions
  * @{
  */
/**
  * @brief  configure the key wake-up settings.
  * @param  TSB_KWUPx:Select the key wake-up unit.
  *   This parameter can be one of the following values:
  *         TSB_KWUPA,TSB_KWUPB.
  * @param  Settings: the pointer to the structure containing the key wake-up settings.
  *         Settings->KeyN : key input select.
  *         Settings->PullUpCtrl: selected static pull-up or dynamic pull-up
  *         Settings->ActiveState: active status setting
  *         Settings->INTNewState: disable /enable KWUP interrupt input
  * @retval None.
  */
void KWUP_SetConfig(TSB_KWUP_TypeDef * TSB_KWUPx, KWUP_SettingTypeDef * Settings)
{
    uint32_t tmp = 0U;
    uint32_t regval = 0U;
    uint32_t tmpAddr = 0U;
    assert_param(IS_KWUP_ALL_UNIT(TSB_KWUPx));
    assert_param(IS_POINTER_NOT_NULL(Settings));
    assert_param(IS_KWUP_PULL_UP_CTRL(Settings->PullUpCtrl));
    assert_param(IS_KWUP_ACTIVE_STATE(Settings->ActiveState));
    assert_param(IS_FUNCTIONAL_STATE(Settings->INTNewState));
    if (TSB_KWUPx == TSB_KWUPA) {
        assert_param(IS_KWUPA_INPUT(Settings->KeyN));
    } else {
        assert_param(IS_KWUPB_INPUT(Settings->KeyN));
    }

    tmp = (uint32_t) Settings->INTNewState;
    tmp |= ((uint32_t) Settings->ActiveState) << 4U;
    tmp |= ((uint32_t) Settings->PullUpCtrl) << 7U;

    tmpAddr = (uint32_t) TSB_KWUPx;
    tmpAddr += (uint32_t) Settings->KeyN * 4U;
    regval = (*((uint32_t *) (tmpAddr)) & KWUP_CRn_MASK);
    regval |= tmp;
    (*((uint32_t *) (tmpAddr))) = regval;

}

/**
  * @brief  get port status sampled in dynamic pull-up period.
  * @param  TSB_KWUPx:Select the key wake-up unit.
  *   This parameter can be one of the following values:
  *         TSB_KWUPA,TSB_KWUPB. 
  * @retval port status.
  */
KWUP_PortStatus KWUP_GetPortStatus(TSB_KWUP_TypeDef * TSB_KWUPx)
{
    KWUP_PortStatus state = { 0U };
    assert_param(IS_KWUP_ALL_UNIT(TSB_KWUPx));
    if (TSB_KWUPx == TSB_KWUPA) {
        state.All = TSB_KWUPx->PKEY;
    } else {
        state.All = TSB_KWUPx->PKEY & (KWUPB_PKEY_MASK);
    }
    return state;
}

/**
  * @brief  Set Dynamic pull-up cycle and Dynamic pull-up period.
  * @param  TSB_KWUPx:Select the key wake-up unit.
  *   This parameter can be one of the following values:
  *         TSB_KWUPA,TSB_KWUPB.
  * @param  T1: activating period when using dynamic pull-up.
  *             This parameter can be one of the following values:
  *             KWUP_CYCLES_2_FS, KWUP_CYCLES_4_FS, KWUP_CYCLES_8_FS, 
  *             KWUP_CYCLES_16_FS,
  * @param  T2: repeated dynamic pull-up operation cycles.
  *             This parameter can be one of the following values:
  *             KWUP_CYCLES_256_FS, KWUP_CYCLES_512_FS,
  *             KWUP_CYCLES_1024_FS, KWUP_CYCLES_2048_FS
  * @retval None.
  */
void KWUP_SetPullUpConfig(TSB_KWUP_TypeDef * TSB_KWUPx, KWUP_PullUpCycles T1, KWUP_PullUpCycles T2)
{
    uint32_t tmp = 0U;
    assert_param(IS_KWUP_ALL_UNIT(TSB_KWUPx));
    assert_param(IS_KWUP_PULL_UP_PERIOD_T1(T1));
    assert_param(IS_KWUP_PULL_UP_CYCLES_T2(T2));

    tmp = TSB_KWUPx->CNT & KWUP_PULL_UP_CYCLE_MASK;

    tmp |= (uint32_t) ((uint32_t) T1 << 2U);
    tmp |= ((uint32_t) T2 - (uint32_t) KWUP_CYCLES_256_FS) << 4U;
    TSB_KWUPx->CNT = tmp;
}

/**
  * @brief  get key interrupt status 
  * @param  TSB_KWUPx:Select the key wake-up unit.
  *   This parameter can be one of the following values:
  *         TSB_KWUPA,TSB_KWUPB. 
  * @retval key interrupt status.
  */
KWUP_INTStatus KWUP_GetINTStatus(TSB_KWUP_TypeDef * TSB_KWUPx)
{
    KWUP_INTStatus state = { 0U };
    assert_param(IS_KWUP_ALL_UNIT(TSB_KWUPx));
    if (TSB_KWUPx == TSB_KWUPA) {
        state.All = TSB_KWUPx->INT;
    } else {
        state.All = TSB_KWUPx->INT & (KWUPB_INT_MASK);
    }
    return state;
}

/** @} */
/* End of group KWUP_Exported_Functions */

/** @} */
/* End of group KWUP */

/** @} */
/* End of group TX04_Periph_Driver */

#endif                          /* defined(__TMPM440_KWUP_H) */
