/**
 *******************************************************************************
 * @file    tmpm440_psc.c
 * @brief   This file provides API functions for PSC driver.
 * @version V2.0.2.3
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm440_psc.h"

#if defined(__TMPM440_PSC_H)

/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @defgroup PSC 
  * @brief PSC driver modules
  * @{
  */

/** @defgroup PSC_Private_Defines
  * @{
  */

#define PSC_SGN_MASK                   ((uint32_t)0x007F0000)
#define PSC_PSCFLG_MASK                ((uint32_t)0x01030000)
#define PSC_RFLG_OVERRUN_MASK          ((uint32_t)0xFFFF0000)
#define PSC_RFLG_STARTUP_MASK          ((uint32_t)0x0000FFFF)

#define PSC_CNT_BRK_SET                ((uint32_t)0x02000000)
#define PSC_CNT_BRK_CLEAR              ((uint32_t)0xFDFFFFFF)
#define PSC_CNT_STEP_SET               ((uint32_t)0x01000000)
#define PSC_CNT_STEP_CLEAR             ((uint32_t)0xFEFFFFFF)
#define PSC_CNT_START_SET              ((uint32_t)0x00010000)
#define PSC_CNT_START_CLEAR            ((uint32_t)0xFFFEFFFF)

/** @} */
/* End of group PSC_Private_Defines */

/** @defgroup PSC_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group PSC_Private_FunctionPrototypes */

/** @defgroup PSC_Private_Functions
  * @{
  */

/** @} */
/* End of group PSC_Private_Functions */

/** @defgroup PSC_Exported_Functions
  * @{
  */

/**
  * @brief  Set to Accumulator.
  * @param  UA0: Value of 32bit.
  * @retval None
  */
void PSC_SetAccumulator(uint32_t UA0)
{
    TSB_PSC->UA0 = UA0;
}

/**
  * @brief  Get Accumulator
  * @param  None
  * @retval Accumulator value
  */
uint32_t PSC_GetAccumulator(void)
{
    uint32_t retval = 0U;

    retval = TSB_PSC->UA0;

    return (retval);
}

/**
  * @brief  Set to Multiplier register
  * @param  UM0: Value of 32bit.
  * @retval None.
  */
void PSC_SetMultiplReg(uint32_t UM0)
{
    TSB_PSC->UM0 = UM0;
}

/**
  * @brief  Get Multiplier register
  * @param  None
  * @retval Multiplier register value
  */
uint32_t PSC_GetMultiplReg(void)
{
    uint32_t retval = 0U;

    retval = TSB_PSC->UM0;

    return (retval);
}

/**
  * @brief  Set to Shift count register
  * @param  UM1: Value of 32bit.
  * @retval None.
  */
void PSC_SetShiftCountReg(uint32_t UM1)
{
    TSB_PSC->UM1 = UM1;
}

/**
  * @brief  Get Shift count register
  * @param  None
  * @retval Shift count register value
  */
uint32_t PSC_GetShiftCountReg(void)
{
    uint32_t retval = 0U;

    retval = TSB_PSC->UM1;

    return (retval);
}

/**
  * @brief  Set to Upper limit value register
  * @param  UL0: Value of 32bit.
  * @retval None.
  */
void PSC_SetUpperLimitReg(uint32_t UL0)
{
    TSB_PSC->UL0 = UL0;
}

/**
  * @brief  Get Upper limit value register
  * @param  None
  * @retval Upper limit value register value
  */
uint32_t PSC_GetUpperLimitReg(void)
{
    uint32_t retval = 0U;

    retval = TSB_PSC->UL0;

    return (retval);
}

/**
  * @brief  Set to Lower limit value register
  * @param  UL1: Value of 32bit.
  * @retval None.
  */
void PSC_SetLowerLimitReg(uint32_t UL1)
{
    TSB_PSC->UL1 = UL1;
}

/**
  * @brief  Get Lower limit value register
  * @param  None
  * @retval Lower limit value register value
  */
uint32_t PSC_GetLowerLimitReg(void)
{
    uint32_t retval = 0U;

    retval = TSB_PSC->UL1;

    return (retval);
}

/**
  * @brief  Set to Add sub value register0
  * @param  UR0: Value of 32bit.
  * @retval None.
  */
void PSC_SetAddSubReg0(uint32_t UR0)
{
    TSB_PSC->UR0 = UR0;
}

/**
  * @brief  Get Add sub value register0
  * @param  None
  * @retval Add sub value register0 value
  */
uint32_t PSC_GetAddSubReg0(void)
{
    uint32_t retval = 0U;

    retval = TSB_PSC->UR0;

    return (retval);
}

/**
  * @brief  Set to Add sub value register1
  * @param  UR1: Value of 32bit.
  * @retval None.
  */
void PSC_SetAddSubReg1(uint32_t UR1)
{
    TSB_PSC->UR1 = UR1;
}

/**
  * @brief  Get Add sub value register1
  * @param  None
  * @retval Add sub value register1 value
  */
uint32_t PSC_GetAddSubReg1(void)
{
    uint32_t retval = 0U;

    retval = TSB_PSC->UR1;

    return (retval);
}

/**
  * @brief  Set to Arithmetic parameter signed register
  * @param  Param: The union that indicates arithmetic parameter signed register
  * @retval None.
  */
void PSC_SetArithmeticParameter(PSC_ArithmeticPARAM Param)
{
    TSB_PSC->SGN = Param.All & PSC_SGN_MASK;
}

/**
  * @brief  Get Arithmetic parameter signed register
  * @param  None
  * @retval The union that indicates arithmetic parameter signed register
  */
PSC_ArithmeticPARAM PSC_GetArithmeticParameter(void)
{
    PSC_ArithmeticPARAM retval = { 0U };

    retval.All = TSB_PSC->SGN;

    return (retval);
}

/**
  * @brief  Set to Address pointer0  
  * @param  AP0: Value of 32bit.
  * @retval None.
  */
void PSC_SetAddrPointer0(uint32_t AP0)
{
    TSB_PSC->AP0 = AP0;
}

/**
  * @brief  Get Address pointer0
  * @param  None 
  * @retval Address pointer0 value
  */
uint32_t PSC_GetAddrPointer0(void)
{
    uint32_t retval = 0U;

    retval = TSB_PSC->AP0;

    return (retval);
}

/**
  * @brief  Set to Address pointer1 
  * @param  AP1: Value of 32bit.
  * @retval None.
  */
void PSC_SetAddrPointer1(uint32_t AP1)
{
    TSB_PSC->AP1 = AP1;
}

/**
  * @brief  Get Address pointer1
  * @param  None
  * @retval Address pointer1 value
  */
uint32_t PSC_GetAddrPointer1(void)
{
    uint32_t retval = 0U;

    retval = TSB_PSC->AP1;

    return (retval);
}

/**
  * @brief  Set to Address pointer2  
  * @param  AP2: Value of 32bit.
  * @retval None.
  */
void PSC_SetAddrPointer2(uint32_t AP2)
{
    TSB_PSC->AP2 = AP2;
}

/**
  * @brief  Get Address pointer2
  * @param  None
  * @retval Address pointer2 value
  */
uint32_t PSC_GetAddrPointer2(void)
{
    uint32_t retval = 0U;

    retval = TSB_PSC->AP2;

    return (retval);
}

/**
  * @brief  Set to Address pointer3
  * @param  AP3: Value of 32bit.
  * @retval None.
  */
void PSC_SetAddrPointer3(uint32_t AP3)
{
    TSB_PSC->AP3 = AP3;
}

/**
  * @brief  Get Address pointer3
  * @param  None
  * @retval Address pointer3 value
  */
uint32_t PSC_GetAddrPointer3(void)
{
    uint32_t retval = 0U;

    retval = TSB_PSC->AP3;

    return (retval);
}

/**
  * @brief  Set to Break pointer
  * @param  BR0: Value of 32bit.
  * @retval None.
  */
void PSC_SetBreakPointer(uint32_t BR0)
{
    TSB_PSC->BR0 = BR0;
}

/**
  * @brief  Get Break pointer
  * @param  None
  * @retval Break pointer value
  */
uint32_t PSC_GetBreakPointer(void)
{
    uint32_t retval = 0U;

    retval = TSB_PSC->BR0;

    return (retval);
}

/**
  * @brief  Set to Program pointer
  * @param  PG0: Value of 32bit.
  * @retval None.
  */
void PSC_SetProgramPointer(uint32_t PG0)
{
    TSB_PSC->PG0 = PG0;
}

/**
  * @brief  Get Program pointer
  * @param  None
  * @retval Program pointer value
  */
uint32_t PSC_GetProgramPointer(void)
{
    uint32_t retval = 0U;

    retval = TSB_PSC->PG0;

    return (retval);
}

/**
  * @brief  Set to Repeat process vector pointer
  * @param  VG0: Value of 32bit.
  * @retval None.
  */
void PSC_SetRepetProcVectPointer(uint32_t VG0)
{
    TSB_PSC->VG0 = VG0;
}

/**
  * @brief  Get Repeat process vector pointer
  * @param  None
  * @retval Repeat process vector pointer value
  */
uint32_t PSC_GetRepetProcVectPointer(void)
{
    uint32_t retval = 0U;

    retval = TSB_PSC->VG0;

    return (retval);
}

/**
  * @brief  Enable or disable Break function
  * @param  NewState: Specify Break function state.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.
  * @retval None.
  */
void PSC_SetBreakFUNC(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        /* Set PSCCNT<BRK> to enable Break function */
        TSB_PSC->CNT |= PSC_CNT_BRK_SET;
    } else {
        /* Clear PSCCNT<BRK> to disable Break function */
        TSB_PSC->CNT &= PSC_CNT_BRK_CLEAR;
    }
}

/**
  * @brief  Enable or disable Step function
  * @param  NewState: Specify Step function state.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.
  * @retval None.
  */
void PSC_SetStepFUNC(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        /* Set PSCCNT<STEP> to enable Step function */
        TSB_PSC->CNT |= PSC_CNT_STEP_SET;
    } else {
        /* Clear PSCCNT<STEP> to disable Step function */
        TSB_PSC->CNT &= PSC_CNT_STEP_CLEAR;
    }
}

/**
  * @brief  Enable or disable Program execution
  * @param  NewState: Specify Program execution state.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.
  * @retval None.
  */
void PSC_SetExecutionFUNC(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        /* Set PSCCNT<START> to enable Program execution */
        TSB_PSC->CNT |= PSC_CNT_START_SET;
    } else {
        /* Clear PSCCNT<START> to disable Program execution */
        TSB_PSC->CNT &= PSC_CNT_START_CLEAR;
    }
}

/**
  * @brief  Set to Arithmetic result register
  * @param  ArithmeticResult: The union that indicates arithmetic result register
  * @retval None.
  */
void PSC_SetArithmeticResult(PSC_ArithmeticResult ArithmeticResult)
{
    TSB_PSC->FLG = ArithmeticResult.All & PSC_PSCFLG_MASK;
}

/**
  * @brief  Get Arithmetic result register
  * @param  None
  * @retval The union that indicates arithmetic result register
  */
PSC_ArithmeticResult PSC_GetArithmeticResult(void)
{
    PSC_ArithmeticResult retval = { 0U };

    retval.All = TSB_PSC->FLG;

    return (retval);
}

/**
  * @brief  Enable or disable the startup event trigger
  * @param  Event: Select the number of startup trigger event
  *   This parameter can be one of the following values or their combination:
  *   PSC_TRIGGER_EVENT_0,  PSC_TRIGGER_EVENT_1,  PSC_TRIGGER_EVENT_2,
  *   PSC_TRIGGER_EVENT_3,  PSC_TRIGGER_EVENT_4,  PSC_TRIGGER_EVENT_5,
  *   PSC_TRIGGER_EVENT_6,  PSC_TRIGGER_EVENT_7,  PSC_TRIGGER_EVENT_8,
  *   PSC_TRIGGER_EVENT_9,  PSC_TRIGGER_EVENT_10, PSC_TRIGGER_EVENT_11,
  *   PSC_TRIGGER_EVENT_12, PSC_TRIGGER_EVENT_13, PSC_TRIGGER_EVENT_14,
  *   PSC_TRIGGER_EVENT_15, PSC_TRIGGER_EVENT_ALL.
  * @param  NewState: Specify startup event trigger state.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.
  * @retval None.
  */
void PSC_SetTriggerEventFUNC(uint32_t Event, FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_PSC_TRIGGER_EVENT(Event));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        TSB_PSC->REN |= Event;
    } else {
        TSB_PSC->REN &= ~Event;
    }
}

/**
  * @brief  Get the startup event trigger state
  * @param  Event: Select the number of startup trigger event
  *   This parameter can be one of the following values:   
  *   PSC_TRIGGER_EVENT_0,  PSC_TRIGGER_EVENT_1,  PSC_TRIGGER_EVENT_2,  PSC_TRIGGER_EVENT_3,
  *   PSC_TRIGGER_EVENT_4,  PSC_TRIGGER_EVENT_5,  PSC_TRIGGER_EVENT_6,  PSC_TRIGGER_EVENT_7,
  *   PSC_TRIGGER_EVENT_8,  PSC_TRIGGER_EVENT_9,  PSC_TRIGGER_EVENT_10, PSC_TRIGGER_EVENT_11,
  *   PSC_TRIGGER_EVENT_12, PSC_TRIGGER_EVENT_13, PSC_TRIGGER_EVENT_14, PSC_TRIGGER_EVENT_15.
  * @retval State of the startup event trigger
  *   The value returned can be one of the following values:
  *   ENABLE or DISABLE
  */
FunctionalState PSC_GetTriggerEventState(uint32_t Event)
{
    FunctionalState retval = DISABLE;
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_PSC_TRIGGER_SEL_EVENT(Event));

    tmp = TSB_PSC->REN & Event;
    if (tmp == Event) {
        retval = ENABLE;
    } else {
        /* Do nothing */
    }

    return (retval);
}

/**
  * @brief  Select the edge of startup trigger event 8 to 11
  * @param  Event: Select the number of startup trigger
  *   This parameter can be one of the following values or their combination:
  *   PSC_TRIGGER_EVENT_8, PSC_TRIGGER_EVENT_9, PSC_TRIGGER_EVENT_10, PSC_TRIGGER_EVENT_11.
  * @param  Edge: Specify the edge of startup trigger event 8 to 11.
  *   This parameter can be one of the following values:
  *   PSC_START_TRIGGER_FALLING or PSC_START_TRIGGER_RISING.
  * @retval None.
  */
void PSC_SetTriggerEventEdge(uint32_t Event, PSC_TriggerEventEdge Edge)
{
    /* Check the parameters */
    assert_param(IS_PSC_TRIGGER_EDGE_EVENT(Event));
    assert_param(IS_PSC_START_TRIGGER_EDGE(Edge));

    if (Edge == PSC_START_TRIGGER_RISING) {
        TSB_PSC->REN |= Event << 8U;
    } else {
        TSB_PSC->REN &= ~(Event << 8U);
    }
}

/**
  * @brief  Get the edge of startup trigger event 8 to 11
  * @param  Event: Selects the number of startup trigger event
  *   This parameter can be one of the following values:
  *   PSC_TRIGGER_EVENT_8, PSC_TRIGGER_EVENT_9, PSC_TRIGGER_EVENT_10, PSC_TRIGGER_EVENT_11.
  * @retval The edge of startup trigger event 8 to 11.
  *   The value returned can be one of the following values:
  *   PSC_START_TRIGGER_FALLING or PSC_START_TRIGGER_RISING
  */
PSC_TriggerEventEdge PSC_GetTriggerEventEdge(uint32_t Event)
{
    PSC_TriggerEventEdge retval = PSC_START_TRIGGER_FALLING;
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_PSC_TRIGGER_EDGE_EVENT_SEL(Event));

    tmp = TSB_PSC->REN & (Event << 8U);
    if (tmp == (Event << 8U)) {
        retval = PSC_START_TRIGGER_RISING;
    } else {
        /* Do nothing */
    }

    return (retval);
}

/**
  * @brief  Get overrun flag caused by the startup trigger event No.15 to 0.
  * @param  None
  * @retval The union that indicates overrun flag.
  */
PSC_OverRunFlag PSC_GetOverRunFlag(void)
{
    uint32_t tmp = 0U;
    PSC_OverRunFlag retval = { 0U };

    tmp = TSB_PSC->RFLG;
    tmp &= PSC_RFLG_OVERRUN_MASK;
    retval.All = tmp;

    return (retval);
}

/**
  * @brief  Get startup event occurrence flag caused by the startup trigger event No.15 to 0.
  * @param  None
  * @retval The union that indicates startup event occurrence flag.
  */
PSC_StartupFlag PSC_GetStartupFlag(void)
{
    uint32_t tmp = 0U;
    PSC_StartupFlag retval = { 0U };

    tmp = TSB_PSC->RFLG;
    tmp &= PSC_RFLG_STARTUP_MASK;
    retval.All = tmp;

    return (retval);
}

/**
  * @brief  Clear the startup event tag/overrun flag caused by No.15 to 0.
  * @param  Event: Select the number of startup trigger event
  *   This parameter can be one of the following values or their combination:
  *   PSC_TRIGGER_EVENT_0,  PSC_TRIGGER_EVENT_1,  PSC_TRIGGER_EVENT_2,
  *   PSC_TRIGGER_EVENT_3,  PSC_TRIGGER_EVENT_4,  PSC_TRIGGER_EVENT_5,
  *   PSC_TRIGGER_EVENT_6,  PSC_TRIGGER_EVENT_7,  PSC_TRIGGER_EVENT_8,
  *   PSC_TRIGGER_EVENT_9,  PSC_TRIGGER_EVENT_10, PSC_TRIGGER_EVENT_11,
  *   PSC_TRIGGER_EVENT_12, PSC_TRIGGER_EVENT_13, PSC_TRIGGER_EVENT_14,      
  *   PSC_TRIGGER_EVENT_15, PSC_TRIGGER_EVENT_ALL. 
  * @retval None.
  */
void PSC_ClearEventOverrunFlag(uint32_t Event)
{
    /* Check the parameters */
    assert_param(IS_PSC_TRIGGER_EVENT(Event));

    TSB_PSC->RCLR |= Event;
}

/**
  * @brief  Set the startup trigger events of peripheral functions by CPU.
  * @param  Event: Select the number of startup trigger event
  *   This parameter can be one of the following values:   
  *   PSC_TRIGGER_EVENT_0,  PSC_TRIGGER_EVENT_1,  PSC_TRIGGER_EVENT_2,  PSC_TRIGGER_EVENT_3,
  *   PSC_TRIGGER_EVENT_4,  PSC_TRIGGER_EVENT_5,  PSC_TRIGGER_EVENT_6,  PSC_TRIGGER_EVENT_7,
  *   PSC_TRIGGER_EVENT_8,  PSC_TRIGGER_EVENT_9,  PSC_TRIGGER_EVENT_10, PSC_TRIGGER_EVENT_11,
  *   PSC_TRIGGER_EVENT_12, PSC_TRIGGER_EVENT_13, PSC_TRIGGER_EVENT_14, PSC_TRIGGER_EVENT_15.
  * @retval None.
  */
void PSC_SetStartup(uint32_t Event)
{
    /* Check the parameters */
    assert_param(IS_PSC_TRIGGER_SEL_EVENT(Event));

    TSB_PSC->RSET |= Event;
}

/**
  * @brief  Set output data for PSC.
  * @param  Data: Specified value will be written to PSCPTOUT register.
  * @retval None.
  */
void PSC_SetOutputData(uint8_t Data)
{
    TSB_PSC->PTOUT = Data;
}

/**
  * @brief  Set output data bit for PSC.
  * @param  Bit_x: Select PSC output bit.
  *   This parameter can be one of the following values or their combination:
  *   PSC_BIT_0, PSC_BIT_1, PSC_BIT_2, PSC_BIT_3, PSC_BIT_4,
  *   PSC_BIT_5, PSC_BIT_6, PSC_BIT_7, PSC_BIT_ALL
  * @param BitValue: The value of specified Bit.
  *   This parameter can be one of the following values:
  *   PSC_BIT_VALUE_0, PSC_BIT_VALUE_1
  * @retval None.
  */
void PSC_SetOutputDataBit(uint8_t Bit_x, uint8_t BitValue)
{
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_PSC_BIT_VALUE(BitValue));

    if (BitValue) {
        tmp = Bit_x;
        TSB_PSC->PTOUT |= (uint32_t) tmp;
    } else {
        tmp = ~Bit_x;
        TSB_PSC->PTOUT &= (uint32_t) tmp;
    }
}

/**
  * @brief  Enable or disable data output for PSC.
  * @param  Bit_x: Select PSC output bit.
  *   This parameter can be one of the following values or their combination:
  *   PSC_BIT_0, PSC_BIT_1, PSC_BIT_2, PSC_BIT_3, PSC_BIT_4, 
  *   PSC_BIT_5, PSC_BIT_6, PSC_BIT_7, PSC_BIT_ALL,
  * @param  NewState: Specify data output state.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.
  * @retval None.
  */
void PSC_SetDataOutputFUNC(uint8_t Bit_x, FunctionalState NewState)
{
    uint8_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    if (NewState == ENABLE) {
        tmp = Bit_x;
        TSB_PSC->PTCR |= (uint32_t) tmp;
    } else {
        tmp = ~Bit_x;
        TSB_PSC->PTCR &= (uint32_t) tmp;
    }
}

/**
  * @brief  Get the state of data output for PSC.
  * @param  Bit_x: Select PSC output bit.
  *   This parameter can be one of the following values:
  *   PSC_BIT_0, PSC_BIT_1, PSC_BIT_2, PSC_BIT_3, 
  *   PSC_BIT_4, PSC_BIT_5, PSC_BIT_6, PSC_BIT_7.
  * @retval State of data output. 
  *   The value returned can be one of the following values:
  *   ENABLE or DISABLE
  */
FunctionalState PSC_GetDataOutputState(uint8_t Bit_x)
{
    uint32_t tmp1 = 0U;
    uint32_t tmp2 = 0U;
    FunctionalState retval = DISABLE;

    /* Check the parameters */
    assert_param(IS_PSC_BIT(Bit_x));

    tmp1 = (uint32_t) Bit_x;
    tmp2 = TSB_PSC->PTCR & tmp1;
    if (tmp1 == tmp2) {
        retval = ENABLE;
    } else {
        /* Do nothing */
    }
    return (retval);
}

/**
  * @brief  Get input data for PSC.
  * @param  None
  * @retval Input data.
  */
uint8_t PSC_GetInputData(void)
{
    uint8_t retval = 0U;

    retval = (uint8_t) TSB_PSC->PTIN;
    return (retval);
}

/**
  * @brief  Get input data bit for PSC.
  * @param  Bit_x: Select PSC input bit.
  *   This parameter can be one of the following values:
  *   PSC_BIT_0, PSC_BIT_1, PSC_BIT_2, PSC_BIT_3, 
  *   PSC_BIT_4, PSC_BIT_5, PSC_BIT_6, PSC_BIT_7.
  * @retval Value of input data bit. 
  *   The value returned can be one of the following values:
  *   PSC_BIT_VALUE_1 or PSC_BIT_VALUE_0
  */
uint8_t PSC_GetInputDataBit(uint8_t Bit_x)
{
    uint32_t tmp1 = 0U;
    uint8_t tmp2 = 0U;
    uint8_t retval = PSC_BIT_VALUE_0;

    /* Check the parameters */
    assert_param(IS_PSC_BIT(Bit_x));

    tmp1 = TSB_PSC->PTIN;
    tmp2 = (uint8_t) tmp1 & Bit_x;
    if (tmp2 == Bit_x) {
        retval = PSC_BIT_VALUE_1;
    } else {
        /* Do nothing */
    }
    return (retval);
}

/**
  * @brief  Get the state of PSC program execution.
  * @param  None
  * @retval The state of PSC program execution.
  *   The value returned can be one of the following values:
  *   PSC_PROGRAM_STOP or PSC_PROGRAM_EXECUTE
  */
PSC_ExecutionState PSC_GetExecutionState(void)
{
    PSC_ExecutionState retval = PSC_PROGRAM_STOP;
    uint32_t tmp = 0U;

    tmp = TSB_PSC->CNT & PSC_CNT_START_SET;
    if (tmp == PSC_CNT_START_SET) {
        retval = PSC_PROGRAM_EXECUTE;
    } else {
        /* Do nothing */
    }

    return (retval);
}

/** @} */
/* End of group PSC_Exported_Functions */

/** @} */
/* End of group PSC */

/** @} */
/* End of group TX04_Periph_Driver */

#endif                          /* defined(__TMPM440_PSC_H) */
