/**
 *******************************************************************************
 * @file    tmpm440_tmrc.c
 * @brief   This file provides API functions for TMRC driver.
 * @version V2.0.2.4
 * @date    2014/05/20
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 *
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm440_tmrc.h"

#if defined (__TMPM440_TMRC_H)
/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @defgroup TMRC
  * @brief TMRC driver modules
  * @{
  */

/** @defgroup TMRC_Private_Defines
  * @{
  */

#define EN_TCEN_SET           ((uint32_t)0x00000080)
#define EN_TCEN_CLEAR         ((uint32_t)0xFFFFFF7F)

#define TBTRUN_BIT3_CLEAR     ((uint32_t)0xFFFFFFF7)
#define TBTRUN_RUN_SET        ((uint32_t)0x00000003)
#define TBTRUN_RUN_CLEAR      ((uint32_t)0xFFFFFFFC)
#define TBTRUN_TBTCAP_SET     ((uint32_t)0x00000004)
#define TBTRUN_TBTCAP_CLEAR   ((uint32_t)0xFFFFFFFB)
#define TBTRUN_I2TB_SET       ((uint32_t)0x00000040)
#define TBTRUN_I2TB_CLEAR     ((uint32_t)0xFFFFFFBF)

#define TBTCR_BIT456_CLEAR    ((uint32_t)0xFFFFFF8F)
#define TBTCR_TBTCLK_CLEAR    ((uint32_t)0xFFFFFFF0)
#define TBTCR_TBTNF_SET       ((uint32_t)0x00000080)
#define TBTCR_TBTNF_CLEAR     ((uint32_t)0xFFFFFF7F)

#define TMRC_CMPCTRLn_MASK    ((uint32_t)0xFFFFFF8C)
#define TMRC_CAPCRn_MASK      ((uint32_t)0xFFFFFF7C)

/** @} */
/* End of group TMRC_Private_Defines */

/** @defgroup TMRC_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group TMRC_Private_FunctionPrototypes */

/** @defgroup TMRC_Private_Functions
  * @{
  */

/** @} */
/* End of group TMRC_Private_Functions */

/** @defgroup TMRC_Exported_Functions
  * @{
  */

/**
  * @brief  Enable the specified TMRC channel.
  * @param  TCx: Select the TMRC channel.
  *   This parameter can be one of the following values:
  *   TSB_TC.
  * @retval None
  */
void TMRC_Enable(TSB_TC_TypeDef * TCx)
{
    /* Check the parameters */
    assert_param(IS_TMRC_ALL_PERIPH(TCx));
    /* Set TCxEN<TCEN> to enable TCx */
    TCx->EN |= EN_TCEN_SET;
}

/**
  * @brief  Disable the specified TMRC channel.
  * @param  TCx: Select the TMRC channel.
  *   This parameter can be one of the following values:
  *   TSB_TC.
  * @retval None
  */
void TMRC_Disable(TSB_TC_TypeDef * TCx)
{
    /* Check the parameters */
    assert_param(IS_TMRC_ALL_PERIPH(TCx));
    /* Clear TCxEN<TCEN> to disable TCx */
    TCx->EN &= EN_TCEN_CLEAR;
}


/**
  * @brief  Start or stop counter and prescaler of the specified TMRC channel.
  * @param  TCx: Select the TMRC channel.
  *   This parameter can be one of the following values:
  *   TSB_TC.
  * @param  Cmd: The command for the counter and prescaler.
  *   This parameter can be TMRC_RUN or TMRC_STOP.
  * @retval None
  */
void TMRC_SetRunState(TSB_TC_TypeDef * TCx, uint32_t Cmd)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_TMRC_ALL_PERIPH(TCx));
    assert_param(IS_TMRC_CMD(Cmd));

    tmp = TCx->TBTRUN;
    tmp &= TBTRUN_BIT3_CLEAR;
    if (Cmd == TMRC_RUN) {
        /* Set TCxTBTRUN<TBTPRUN><TBTRUN> to run specified TMRC channel */
        TCx->TBTRUN = tmp | TBTRUN_RUN_SET;
    } else {
        /* Set TCxTBTRUN<TBTPRUN><TBTRUN> to stop and clear specified TMRC channel */
        TCx->TBTRUN = tmp & TBTRUN_RUN_CLEAR;
    }
}

/**
  * @brief  Enable or disable TMRC when system is in idle mode.
  * @param  TCx: Select the TMRC channel.
  *   This parameter can be one of the following values:
  *   TSB_TC.
  * @param  NewState: New state of TMRC in system idle mode.
  *   This parameter can be ENABLE or DISABLE.
  * @retval None
  */
void TMRC_SetIdleMode(TSB_TC_TypeDef * TCx, FunctionalState NewState)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_TMRC_ALL_PERIPH(TCx));
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    tmp = TCx->TBTRUN;
    tmp &= TBTRUN_BIT3_CLEAR;
    if (NewState == ENABLE) {
        /* Set TCxTBTRUN<I2TB> to enable TCx in system idle mode */
        tmp |= TBTRUN_I2TB_SET;
    } else {
        /* Clear TCxTBTRUN<I2TB> to disable TCx in system idle mode */
        tmp &= TBTRUN_I2TB_CLEAR;
    }

    TCx->TBTRUN = tmp;
}

/**
  * @brief  Capture counter by software.
  * @param  TCx: Select the TMRC channel.
  *   This parameter can be one of the following values:
  *   TSB_TC.
  * @retval None
  */
void TMRC_ExecuteSWCapture(TSB_TC_TypeDef * TCx)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_TMRC_ALL_PERIPH(TCx));

    tmp = TCx->TBTRUN;
    tmp &= TBTRUN_BIT3_CLEAR;
    /* Set software capture */
    tmp |= TBTRUN_TBTCAP_SET;
    TCx->TBTRUN = tmp;
}

/**
  * @brief  TBT source clock selection.
  * @param  TCx: Select the TMRC channel.
  *   This parameter can be one of the following values:
  *   TSB_TC.
  * @param  ClkDiv: Source clock.
  *   This parameter can be one of the following values:
  *   TMRC_CLK_DIV_4, TMRC_CLK_DIV_8, TMRC_CLK_DIV_16, TMRC_CLK_DIV_32, 
  *   TMRC_CLK_DIV_64, TMRC_CLK_DIV_128, TMRC_CLK_DIV_256, 
  *   TMRC_CLK_DIV_512,TMRC_CLK_TCTBT_IN.
  * @retval None
  */
void TMRC_SetSrcClk(TSB_TC_TypeDef * TCx, uint32_t ClkDiv)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_TMRC_ALL_PERIPH(TCx));
    assert_param(IS_TMRC_CLK_DIV(ClkDiv));

    tmp = TCx->TBTCR;
    tmp &= TBTCR_BIT456_CLEAR;
    tmp &= TBTCR_TBTCLK_CLEAR;
    /* TBT source clock set */
    tmp |= ClkDiv;
    TCx->TBTCR = tmp;
}

/**
  * @brief  TCTBTIN input noise filter.
  * @param  TCx: Select the TMRC channel.
  *   This parameter can be one of the following values:
  *   TSB_TC.
  * @param  NosFlt: Controls noise filter of TCTBTIN pin input.
  *   This parameter can be TMRC_FILTER or TMRC_NOFILTER.
  * @retval None
  */
void TMRC_SetNoiseFilter(TSB_TC_TypeDef * TCx, TMRC_NoiseFilter NosFlt)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_TMRC_ALL_PERIPH(TCx));
    assert_param(IS_TMRC_NOISE_FILTER(NosFlt));

    tmp = TCx->TBTCR;
    tmp &= TBTCR_BIT456_CLEAR;
    if (NosFlt == TMRC_FILTER) {
        /* Set TCxTBTCR<TBTNF> to 1 */
        tmp |= TBTCR_TBTNF_SET;
    } else {
        /* Clear TCxTBTCR<TBTNF> to 0 */
        tmp &= TBTCR_TBTNF_CLEAR;
    }
    /* Input noise filter set */
    TCx->TBTCR = tmp;
}

/**
  * @brief  Get TMRC Software capture value of TCx.
  * @param  TCx: Select the TMRC channel.
  *   This parameter can be one of the following values:
  *   TSB_TC.
  * @retval Capture value of TCxTBTCP<CAP>
  */
uint32_t TMRC_GetSWCaptureValue(TSB_TC_TypeDef * TCx)
{
    uint32_t CapReg = 0U;
    /* Check the parameters */
    assert_param(IS_TMRC_ALL_PERIPH(TCx));
    /* Read the capture value of TCx */
    CapReg = TCx->TBTCP;
    /* Return the capture value of TCx */
    return CapReg;
}

/**
  * @brief  Get TMRC read capture value of TCx.
  * @param  TCx: Select the TMRC channel.
  *   This parameter can be one of the following values:
  *   TSB_TC.
  * @retval Read capture value of TCxTBTRDCP<RDCP>
  */
uint32_t TMRC_GetReadCaptureValue(TSB_TC_TypeDef * TCx)
{
    uint32_t RdCapReg = 0U;
    /* Check the parameters */
    assert_param(IS_TMRC_ALL_PERIPH(TCx));
    /* Read the read capture value of TCx */
    RdCapReg = TCx->TBTRDCP;
    /* Return the read capture value of TCx */
    return RdCapReg;
}

/**
  * @brief  Compare control register configration.
  * @param  TCx: Select the TMRC channel.
  *   This parameter can be one of the following values:
  *   TSB_TC.
  * @param  CMPConfigStruct: The structure containing basic compare control register configuration.
  * @retval None
  */
void TMRC_CMPRegConfig(TSB_TC_TypeDef * TCx, TMRC_CMPConfigTypeDef * CMPConfigStruct)
{
    uint32_t tmp = 0U;
    uint32_t regval = 0U;

    /* Check the parameters */
    assert_param(IS_POINTER_NOT_NULL(CMPConfigStruct));
    assert_param(IS_TMRC_ALL_PERIPH(TCx));
    assert_param(IS_TMRC_CMP_REG(CMPConfigStruct->CMPReg));
    assert_param(IS_TMRC_FF_REVERSE_TRG(CMPConfigStruct->FFReverseTrg));
    assert_param(IS_TMRC_FLIPFLOP_CTRL(CMPConfigStruct->FlipFlopCtrl));
    assert_param(IS_TMRC_CMP_DB_CTRL(CMPConfigStruct->CMPDBCtrl));
    assert_param(IS_TMRC_CMP_MATCH(CMPConfigStruct->CMPMatch));

    tmp = (uint32_t) CMPConfigStruct->CMPMatch;
    tmp |= ((uint32_t) CMPConfigStruct->CMPDBCtrl) << 1U;
    tmp |= ((uint32_t) CMPConfigStruct->FlipFlopCtrl) << 4U;
    tmp |= ((uint32_t) CMPConfigStruct->FFReverseTrg) << 6U;

    switch (CMPConfigStruct->CMPReg) {
    case TMRC_CMP_0:
        regval = TCx->CMPCTL0 & TMRC_CMPCTRLn_MASK;
        regval |= tmp;
        TCx->CMPCTL0 = regval;
        break;
    case TMRC_CMP_1:
        regval = TCx->CMPCTL1 & TMRC_CMPCTRLn_MASK;
        regval |= tmp;
        TCx->CMPCTL1 = regval;
        break;
    case TMRC_CMP_2:
        regval = TCx->CMPCTL2 & TMRC_CMPCTRLn_MASK;
        regval |= tmp;
        TCx->CMPCTL2 = regval;
        break;
    case TMRC_CMP_3:
        regval = TCx->CMPCTL3 & TMRC_CMPCTRLn_MASK;
        regval |= tmp;
        TCx->CMPCTL3 = regval;
        break;
    case TMRC_CMP_4:
        regval = TCx->CMPCTL4 & TMRC_CMPCTRLn_MASK;
        regval |= tmp;
        TCx->CMPCTL4 = regval;
        break;
    case TMRC_CMP_5:
        regval = TCx->CMPCTL5 & TMRC_CMPCTRLn_MASK;
        regval |= tmp;
        TCx->CMPCTL5 = regval;
        break;
    case TMRC_CMP_6:
        regval = TCx->CMPCTL6 & TMRC_CMPCTRLn_MASK;
        regval |= tmp;
        TCx->CMPCTL6 = regval;
        break;
    case TMRC_CMP_7:
        regval = TCx->CMPCTL7 & TMRC_CMPCTRLn_MASK;
        regval |= tmp;
        TCx->CMPCTL7 = regval;
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Set the value for comparing the counter.
  * @param  TCx: Select the TMRC channel.
  *   This parameter can be one of the following values:
  *   TSB_TC.
  * @param  CMPRegNum: Compare register number from 0 to 7.
  *   This parameter can be TMRC_CMP_0, TMRC_CMP_1, TMRC_CMP_2, TMRC_CMP_3, 
  *   TMRC_CMP_4, TMRC_CMP_5, TMRC_CMP_6, TMRC_CMP_7
  * @param  CMPRegVal: The data to be set.
  *   The CMPRegVal range is 0x00000000 to 0xFFFFFFFF.
  * @retval None
  */
void TMRC_SetCMPRegValue(TSB_TC_TypeDef * TCx, TMRC_CMPReg CMPRegNum, uint32_t CMPRegVal)
{
    /* Check the parameters */
    assert_param(IS_TMRC_ALL_PERIPH(TCx));
    assert_param(IS_TMRC_CMP_REG(CMPRegNum));
    /*assert_param(IS_TMRC_CMPVAL(CMPRegVal)); *//* uint32_t */

    switch (CMPRegNum) {
    case TMRC_CMP_0:
        TCx->CMP0 = CMPRegVal;
        break;
    case TMRC_CMP_1:
        TCx->CMP1 = CMPRegVal;
        break;
    case TMRC_CMP_2:
        TCx->CMP2 = CMPRegVal;
        break;
    case TMRC_CMP_3:
        TCx->CMP3 = CMPRegVal;
        break;
    case TMRC_CMP_4:
        TCx->CMP4 = CMPRegVal;
        break;
    case TMRC_CMP_5:
        TCx->CMP5 = CMPRegVal;
        break;
    case TMRC_CMP_6:
        TCx->CMP6 = CMPRegVal;
        break;
    case TMRC_CMP_7:
        TCx->CMP7 = CMPRegVal;
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Get the value of comparing the counter.
  * @param  TCx: Select the TMRC channel.
  *   This parameter can be one of the following values:
  *   TSB_TC.
  * @param  CMPRegNum: Compare register number from 0 to 7.
  *   This parameter can be TMRC_CMP_0, TMRC_CMP_1, TMRC_CMP_2, TMRC_CMP_3, 
  *   TMRC_CMP_4, TMRC_CMP_5, TMRC_CMP_6, TMRC_CMP_7
  * @retval the value of compare register CMP<31:0>.
  */
uint32_t TMRC_GetCMPRegValue(TSB_TC_TypeDef * TCx, TMRC_CMPReg CMPRegNum)
{
    uint32_t CMPRegVal = 0U;

    /* Check the parameters */
    assert_param(IS_TMRC_ALL_PERIPH(TCx));
    assert_param(IS_TMRC_CMP_REG(CMPRegNum));

    switch (CMPRegNum) {
    case TMRC_CMP_0:
        CMPRegVal = TCx->CMP0;
        break;
    case TMRC_CMP_1:
        CMPRegVal = TCx->CMP1;
        break;
    case TMRC_CMP_2:
        CMPRegVal = TCx->CMP2;
        break;
    case TMRC_CMP_3:
        CMPRegVal = TCx->CMP3;
        break;
    case TMRC_CMP_4:
        CMPRegVal = TCx->CMP4;
        break;
    case TMRC_CMP_5:
        CMPRegVal = TCx->CMP5;
        break;
    case TMRC_CMP_6:
        CMPRegVal = TCx->CMP6;
        break;
    case TMRC_CMP_7:
        CMPRegVal = TCx->CMP7;
        break;
    default:
        /* Do nothing */
        break;
    }
    /* Return the value of compare register */
    return CMPRegVal;
}

/**
  * @brief  Capture control register configration.
  * @param  TCx: Select the TMRC channel.
  *   This parameter can be one of the following values:
  *   TSB_TC.
  * @param  CAPConfigStruct: The structure containing basic Capture control register configuration.
  * @retval None
  */
void TMRC_CAPRegConfig(TSB_TC_TypeDef * TCx, TMRC_CAPConfigTypeDef * CAPConfigStruct)
{
    uint32_t tmp = 0U;
    uint32_t regval = 0U;

    /* Check the parameters */
    assert_param(IS_POINTER_NOT_NULL(CAPConfigStruct));
    assert_param(IS_TMRC_ALL_PERIPH(TCx));

    assert_param(IS_TMRC_CAP_REG(CAPConfigStruct->CAPReg));
    assert_param(IS_TMRC_NOISE_FILTER(CAPConfigStruct->NoiseFilter));
    assert_param(IS_TMRC_VALID_EDGE_SEL(CAPConfigStruct->ValidEdgeSel));

    tmp = (uint32_t) CAPConfigStruct->ValidEdgeSel;
    tmp |= ((uint32_t) CAPConfigStruct->NoiseFilter) << 7U;

    switch (CAPConfigStruct->CAPReg) {
    case TMRC_CAP_0:
        regval = TCx->CAPCR0 & TMRC_CAPCRn_MASK;
        regval |= tmp;
        TCx->CAPCR0 = regval;
        break;
    case TMRC_CAP_1:
        regval = TCx->CAPCR1 & TMRC_CAPCRn_MASK;
        regval |= tmp;
        TCx->CAPCR1 = regval;
        break;
    case TMRC_CAP_2:
        regval = TCx->CAPCR2 & TMRC_CAPCRn_MASK;
        regval |= tmp;
        TCx->CAPCR2 = regval;
        break;
    case TMRC_CAP_3:
        regval = TCx->CAPCR3 & TMRC_CAPCRn_MASK;
        regval |= tmp;
        TCx->CAPCR3 = regval;
        break;
    default:
        /* Do nothing */
        break;
    }
}

/**
  * @brief  Get TMRC counter captured value.
  * @param  TCx: Select the TMRC channel.
  *   This parameter can be one of the following values:
  *   TSB_TC.
  * @param  CAPRegNum: Capture register number from 0 to 3.
  *   This parameter can be TMRC_CAP_0, TMRC_CAP_1, TMRC_CAP_2, TMRC_CAP_3.
  * @retval Counter captured value of CAP<CAP31:0>
  */
uint32_t TMRC_GetCntCaptureValue(TSB_TC_TypeDef * TCx, TMRC_CAPReg CAPRegNum)
{
    uint32_t CntCapReg = 0U;

    /* Check the parameters */
    assert_param(IS_TMRC_ALL_PERIPH(TCx));
    assert_param(IS_TMRC_CAP_REG(CAPRegNum));

    /* Read the counter captured value of TCx */
    switch (CAPRegNum) {
    case TMRC_CAP_0:
        CntCapReg = TCx->CAP0;
        break;
    case TMRC_CAP_1:
        CntCapReg = TCx->CAP1;
        break;
    case TMRC_CAP_2:
        CntCapReg = TCx->CAP2;
        break;
    case TMRC_CAP_3:
        CntCapReg = TCx->CAP3;
        break;
    default:
        /* Do nothing */
        break;
    }
    /* Return the captured value of TCx */
    return CntCapReg;
}

/** @} */
/* End of group TMRC_Exported_Functions */

/** @} */
/* End of group TMRC */

/** @} */
/* End of group TX04_Periph_Driver */

#endif                          /* defined(__TMPM440_TMRC_H) */
