/**
 *******************************************************************************
 * @file    main.c
 * @brief   the application functions of CG_ModeSwitch demo for the TOSHIBA
 *          'TMPM440' Device Series 
 * @version V0.103
 * @date    2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
 
/*========================================================= 
This is a simple demo to change the CPU operation mode. 
Only 2 modes(NORMAL and STOP) are supported . 

Key1 is connected to pin Y12(PP6, inner pull up, active low)
 ( used to enter STOP mode )
Key2 is connected to pin R11(PP7, inner pull up, active low)
 (used as interrupt source INT0 to release STOP mode )


Key's action to switch mode as showed below: 
Current mode |        Operation               |     Action        | LED display
  NORMAL     | Key1 is pressed(low active)    |  NORMAL --> STOP  |   LED off
  STOP       | Key2 is pressed(low active)    |  STOP --> NORMAL  |   LED on
========================================================= */ 
 
 
/* Includes ------------------------------------------------------------------*/
#include "main.h"
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define NORMAL_MODE             ((uint8_t)0x00)
#define STOP_MODE               ((uint8_t)0x01)
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Global variables ----------------------------------------------------------*/
uint8_t gCurrentMode = NORMAL_MODE;
/* external variables --------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void Mode_Judgement(void);
void System_Init(void);

/* Main function */
int main(void)
{
    System_Init();

    /* Disable interrupts */
    __disable_irq();
   
    CG_ClearINTReq(CG_INT_SRC_0);
    
    /* Set to use INT0 which is pull up and low active) to wake up system from STOP mode */
    CG_SetSTBYReleaseINTSrc(CG_INT_SRC_0, CG_INT_ACTIVE_STATE_FALLING, ENABLE);
    
    NVIC_ClearPendingIRQ(INT0_IRQn);
    NVIC_EnableIRQ(INT0_IRQn);
    __enable_irq();

    
    /* Initialize Variables */
    gCurrentMode = NORMAL_MODE;

    while (1) {
        Mode_Judgement();  

        switch(gCurrentMode) {
        case NORMAL_MODE:
            LED_On(LED0|LED1|LED2|LED3);
            break;
        case STOP_MODE:
            LED_Off(LED0|LED1|LED2|LED3);

            CG_NormalToStop();

            System_Init();
            gCurrentMode = NORMAL_MODE;
            break;
        default:
            gCurrentMode = NORMAL_MODE;
            break;
        }
    }
}

/**
  * @brief  Judge whether go to STOP_MODE
  * @param  None.
  * @retval None.
  */
void Mode_Judgement(void)
{
    uint8_t tmp = 1U; 
    static uint32_t cnt = 0U;
    
    /* check status of Key1 on PP6 , '0' is key pressed */
    tmp = TSB_PP->DATA & (0x1U << 6U);
    
    if( !tmp ) { 
        cnt++;
        if( cnt >= 10000U) { /* Key1 has been pressed long enough */
            if (gCurrentMode == NORMAL_MODE) {
                gCurrentMode = STOP_MODE;
            } else {
                /* Do nothing */
            }
        } else {
                /* Do nothing */
        }
    } else {
       cnt = 0U;
    } 
}

/**
  * @brief  Initialize system
  * @param  None.
  * @retval None.
  */
void System_Init(void)
{
    /* Initialize system */
    CG_SetSystem();

    /* Configure Key1(PP6), Key2(PP7) as input, with pull up */
    TSB_PP->IE |= 0x03U << 6U ;
    TSB_PP->PUP |= 0x03U << 6U ;
    
    /* Configure INT0 function on pin PP7(Key2) */ 
    TSB_PP->FR3 |= 0x01U << 7U;
    
    /* Configure GPIO for LED */
    LED_Init();

}

#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
/*********************************** END OF FILE ******************************/
