/**
 *******************************************************************************
 * @file    main.c
 * @brief   The application functions of ESIO demo for the
 *          TOSHIBA 'TMPM440' Device Series
 * @version V0.101
 * @date    2014/05/20
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 *
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/*--------------------------------------------------------------------
 This demo will show basic usage of ESIO module in M440.
 
 Connect pin A8(PJ0, EXIO_Tx0) to pin B8(PJ4, ESIO_Rx0),
  the LEDs will blink to show the data is being received.
  
 User can define BITRATE_MIN to see the blink effect.
 
 This demo is compiled by EWARM v6.40 and run on Toshiba M440 Ev board. 
---------------------------------------------------------------------*/


/* include file */
#include "tmpm440_esio.h"
#include "led_NoGPIODriver.h"


/* define the ESIO unit used in this demo */
#define ESIO_x   TSB_ESIO0
/* to check the effect due to too low speed, uncomment the line below */
#define BITRATE_MIN 

#define MAX_BUFSIZE  64U

void ConfigESIO(void);
void ConfigGPIOForESIO(void);

typedef enum { SAME = 1U, NOT_SAME = !SAME } TestResult;
TestResult result = NOT_SAME;
TestResult Buffercompare(const uint8_t * Src, const uint8_t * Dst, uint32_t Len);


uint16_t datTx = 1U;        /* its size is the 'FrameLength' */

uint32_t cntTx = 1U;
uint32_t cntRx = 0U;

uint32_t receive = 0U;
uint8_t Rx_Buf[MAX_BUFSIZE] = { 0U };
uint8_t Tx_Buf[MAX_BUFSIZE] = { 0U };   

/* main function */
int main(void)
{

    uint8_t CompOnce = 0U;
    
    ConfigGPIOForESIO();
    ConfigESIO();
    
    /* initialize LEDs before display something */
    LED_Init();

    /* send the 1st data */
    Tx_Buf[0] = datTx;
    ESIO_SetTxData(ESIO_x, datTx);
    
    while (1) {
        if( CompOnce == 0 ) {   
          if (cntRx == MAX_BUFSIZE) {
               CompOnce = 1;
               result = Buffercompare( Tx_Buf, Rx_Buf, MAX_BUFSIZE );
          } else {
            /* Do nothing */
          }
        } else {
            /* Do nothing */
        }
    }
}

/**
  * @brief  To configure the pins as for ESIO function
  * @param  None
  * @retval None
  */
void ConfigGPIOForESIO(void)
{
    if( ESIO_x == TSB_ESIO0 ) {
        TSB_PJ->CR = 0x0FU;
        TSB_PJ->FR1 = 0xFFU;
        TSB_PJ->IE = 0xF0U; 
        TSB_PK->CR = 0x03U;
        TSB_PK->FR1 = 0x03U;
        TSB_PK->IE = 0x00U;                       
    } else if( ESIO_x == TSB_ESIO1 ) {
        TSB_PL->CR = 0x0FU;
        TSB_PL->FR1 = 0xFFU;
        TSB_PL->IE = 0xF0U;    
        TSB_PM->CR = 0x03U;
        TSB_PM->FR1 = 0x03U;
        TSB_PM->IE = 0x00U;         
    } else {
        TSB_PN->CR = 0x0FU;
        TSB_PN->FR1 = 0xFFU;
        TSB_PN->IE = 0xF0U;
        TSB_PP->CR = 0x03U;
        TSB_PP->FR1 = 0x03U;
        TSB_PP->IE = 0x00U;             
    }
    
}

    
/**
  * @brief  To configure ESIO by using functions from ESIO driver
  * @param  None
  * @retval None
  */
void ConfigESIO(void)
{
    /* Single transfer */
    uint8_t transfer_num = 1;   
    
    /* maximum clock: clk = phiT0/4, divider=1 */
    ESIO_BaudClock clk = ESIO_PHIT0_DIVIDE_4;   
    uint8_t divider = 1U;
    
    ESIO_InitTypeDef init = { 0U };
    
    ESIO_Enable(ESIO_x);
    /* ESIO_SWReset(ESIO_x); */
    
    ESIO_SetTxRxCtrl(ESIO_x, ENABLE);
    ESIO_SelectMode(ESIO_x, ESIO_MODE_SIO);
    ESIO_SelectTransferMode(ESIO_x, ESIO_TRMODE_TXRX);
    
    ESIO_SetTransferNum(ESIO_x, transfer_num);
    
    ESIO_SetFIFOLevelINT(ESIO_x, ESIO_TX , 0U);
    ESIO_SetFIFOLevelINT(ESIO_x, ESIO_RX , 0U);
    
    /* enable ESIO interrupt */
    ESIO_SetINT(ESIO_x, ESIO_INT_ALL, ENABLE);
    
#ifdef BITRATE_MIN
    clk = ESIO_PHIT0_DIVIDE_1024;
    divider = 16U;
#endif 
    ESIO_SetBaudRate(ESIO_x, clk, divider );
    
    init.DataDirection = ESIO_LSB_FIRST;
    init.FrameLength = sizeof(datTx) * 8U;        /* size of datTx in main() */
    init.CycleBetweenFrames = 1U;
    init.NumberOfLINE = 1U;
    init.ClkPolority = ESIO_CLKPOL_LOW;
    init.ShortestIdleCycle = 1U;
    init.CS0ActiveLevel = ESIO_CS_ACTIVE_LOW;
    init.DelayCycleNum_CS_SCLK = 1U;
    init.DelayCycleNum_Negate_CS = 1U;
    init.HorizontalParityCheck = DISABLE;
    /* init.HorizontalParity = ESIO_PARITY_ODD; */
    init.VerticalParityCheck = DISABLE;
    ESIO_Init(ESIO_x, &init);   

    NVIC_ClearPendingIRQ(INTE0RX_IRQn );
    NVIC_EnableIRQ(INTE0RX_IRQn);

    NVIC_ClearPendingIRQ(INTE0TX_IRQn);
    NVIC_EnableIRQ(INTE0TX_IRQn);

    NVIC_ClearPendingIRQ(INTE0ERR_IRQn);
    NVIC_EnableIRQ(INTE0ERR_IRQn);
            
    __enable_irq();
    
        
}

/**
  * @brief  ISR for ESIO Rx
  * @param  None
  * @retval None
  */
void INTE0RX_IRQHandler(void)
{
    uint8_t tmp = 0U;
    
    receive = ESIO_GetRxData(ESIO_x);
    if (cntRx < MAX_BUFSIZE) {
       Rx_Buf[cntRx] = (uint8_t)receive;
       cntRx++;
   }
   /* adjust the loop blinkging speed */
   tmp = (uint8_t) (receive >> 4U);
   LED_Display(tmp);    
}

/**
  * @brief  ISR for ESIO Tx
  * @param  None
  * @retval None
  */
void INTE0TX_IRQHandler(void)
{
    datTx++;
    ESIO_SetTxData(ESIO_x, datTx);
    if (cntTx < MAX_BUFSIZE) {
        Tx_Buf[cntTx] = datTx;
        cntTx++;
    } 
}

/**
  * @brief  ISR for ESIO Error
  * @param  None
  * @retval None
  */
void INTE0ERR_IRQHandler(void)
{
    __NOP();
}


/**
  * @brief  To compare two half word buffer
  * @param  Src : pointer to point source buffer 
  * @param  Dst : pointer to point destination buffer   
  * @param  Len : how many data will be compared 
  * @retval SAME: two buffer contain same data
            NOT_SAME: two buffer contain different data  
  */
TestResult Buffercompare(const uint8_t * Src, const uint8_t * Dst, uint32_t Len)
{
    uint32_t tmp = 0U;
    TestResult result = NOT_SAME;
    
    for( tmp = 0U; tmp < Len ; tmp++ ) {
        if( Src[tmp] != Dst[tmp] ) {
             break;
       } else {
        result = SAME;
       }
    }
    return result;
}   



#ifdef DEBUG
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __NOP();
    }
}
#endif
