/**
 *******************************************************************************
 * @file	cg.c
 * @brief	cg driver for the TOSHIBA 'TMPM440' Device Series
 * @version V0.102
 * @date	2014/05/20
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */
#ifndef  __BIG_ENDIAN
#ifdef __IAR_SYSTEMS_ICC__
#if 1 != __LITTLE_ENDIAN__
#define     __BIG_ENDIAN
#endif
#endif
#endif

/** include files **/
#include "tmpm440_cg.h"

/* Private define ------------------------------------------------------------*/

#define OSCCR_Val               (0x000E0104UL)  /* OSCCR<OSCSEL> = 1, OSCCR<XEN2> = 0, OSCCR<XEN1> = 1, OSCCR<PLL0ON> = 1, OSCCR<PLL1ON> = 0,<EHOSCSEL> = 1 */

#define CG_10M_MUL_5_FPLL       (0x00007026UL<<1U)      /* If input colck is 10MHz, PLL0SET<14:0> = 0x00007026UL, output clock is 50MHz */
#define CG_10M_MUL_6_FPLL       (0x0000782EUL<<1U)      /* If input colck is 10MHz, PLL0SET<14:0> = 0x0000782EUL, output clock is 60MHz */
#define CG_10M_MUL_8_FPLL       (0x000078BEUL<<1U)      /* If input colck is 10MHz, PLL0SET<14:0> = 0x000078BEUL, output clock is 80MHz */
#define CG_10M_MUL_10_FPLL      (0x0000704DUL<<1U)      /* If input colck is 10MHz, PLL0SET<14:0> = 0x0000704DUL, output clock is 100MHz */

#define PLL0SEL_Ready           CG_10M_MUL_10_FPLL
#define PLL0SEL_Val             (PLLSEL_Ready|1U)
#define PLLSEL_MASK             (0x0000FFFEUL)

/* Define clocks */
#define OSC_8M                  ( 8000000UL)
#define OSC_10M                 (10000000UL)
#define OSC_16M                 (16000000UL)

#define EXTALH                  OSC_10M /* External high-speed oscillator freq */
#define XTALH                   OSC_10M /* Internal high-speed oscillator freq */

/* Configure Warm-up time */
#define HZ_1M                   1000000UL
#define WU_TIME_EXT             5000UL  /* warm-up time for EXT is 5ms   */
#define WU_TIME_PLL             100UL   /* warm-up time for PLL is 100us */
#define OSCCR_WUODR_MASK        (0x000FFFFFUL)
#define OSCCR_WUODR_EXT         ((WU_TIME_EXT * EXTALH / HZ_1M / 16U) << 20U)   /* OSCCR<WUPODR11:0> = warm-up time(us) * EXTALH(MHz) / 16 */
#define OSCCR_WUODR_PLL         ((WU_TIME_PLL * EXTALH / HZ_1M / 16U) << 20U)

/* Exported functions ------------------------------------------------------- */
void configCG_FS(uint16_t time);

void configSTOP2(void);
int isStanbyRelease(void);

/**
  * @brief  Enable the fs for KSCAN
  * @param  warme up time
  * @retval None
  */
void configCG_FS(uint16_t time)
{
    volatile WorkState st = BUSY;

    /* Enable Fs */
    CG_SetFs(ENABLE);
    CG_SetWarmUpTime(CG_WARM_UP_SRC_OSC_EXT, time);
    do {
        st = CG_GetWarmUpState();
    } while (st != DONE);
}

/**
  * @brief  Config STOP2 mode
  * @param  None
  * @retval None
  */
void configSTOP2(void)
{
    volatile uint32_t tmp;

    /*Set standby mode as Stop2 */
    CG_SetSTBYMode(CG_STBY_MODE_STOP2);


    TSB_CG->PLLSEL &= 0xfffffffe;
    do {
        tmp = (TSB_CG->PLLSEL & 0x00000001);
    } while (0x00000000 != tmp);

    /* Clock(external) Setup */
    TSB_CG->OSCCR &= OSCCR_WUODR_MASK;
    TSB_CG->OSCCR |= OSCCR_WUODR_EXT;

    if (TSB_CG_OSCCR_XEN2 == 0U) {
        TSB_CG->OSCCR |= 0x00010000;

        TSB_CG->OSCCR |= 0x00080000;

        TSB_CG->OSCCR |= 0x00000001;

        /*wait warm-up finish */
        do {
            tmp = (TSB_CG->OSCCR & 0x00000002);
        } while (0x00000000 != tmp);
    }

    TSB_CG->OSCCR &= 0xfffdffff;

    TSB_CG->STBYCR |= 0x00020000;
}

/**
  * @brief  Check the reset factor
  * @param  None
  * @retval 0 or 1
  */
int isStanbyRelease(void)
{
    uint32_t tmp;

    tmp = TSB_CG->RSTFLG;


    TSB_CG->RSTFLG = 0U;

    if ((tmp & (1U << 1U)) == (1U << 1U))
        return 0;

    if ((tmp & (1U << 3U)) == (1U << 3U))
        return 1;

    return 0;
}
